def detach(argument, target=None, by_id=False):
    r"""
    Detaches indicators-equal-to-``argument`` from ``target``.
        
    When ``target`` is none ``argument`` must be a spanner; spanner will then
    detach from all leaves to which spanner attaches.

    Set ``by_id`` to true to detach exact ``argument`` from ``target`` (rather
    than detaching all indicators-equal-to-``argument``).

    ..  container:: example

        Detaches articulations from first note in staff:

        >>> staff = abjad.Staff("c'4 d' e' f'")
        >>> abjad.attach(abjad.Articulation('>'), staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'4
                -\accent
                d'4
                e'4
                f'4
            }

        >>> abjad.detach(abjad.Articulation, staff[0])
        (Articulation('>'),)
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'4
                d'4
                e'4
                f'4
            }

    ..  container:: example

        The use of ``by_id`` is motivated by the following.

        Consider the three document-specifier markups below:

        >>> markup_1 = abjad.Markup('tutti', direction=abjad.Up)
        >>> markup_2 = abjad.Markup('with the others', direction=abjad.Up)
        >>> markup_3 = abjad.Markup('with the others', direction=abjad.Up)

        Markups two and three compare equal:

        >>> markup_2 == markup_3
        True

        But document-tagging like this makes sense for score and two diferent
        parts:

        >>> staff = abjad.Staff("c'4 d' e' f'")
        >>> abjad.attach(markup_1, staff[0], tag='+SCORE')
        >>> abjad.attach(
        ...     markup_2,
        ...     staff[0],
        ...     deactivate=True,
        ...     tag='+PARTS_VIOLIN_1',
        ...     )
        >>> abjad.attach(
        ...     markup_3,
        ...     staff[0],
        ...     deactivate=True,
        ...     tag='+PARTS_VIOLIN_2',
        ...     )
        >>> abjad.show(staff) # doctest: +SKIP

        >>> abjad.f(staff, strict=50)
        \new Staff
        {
            c'4
            ^ \markup { tutti }                           %! +SCORE
        %@% ^ \markup { "with the others" }               %! +PARTS_VIOLIN_1
        %@% ^ \markup { "with the others" }               %! +PARTS_VIOLIN_2
            d'4
            e'4
            f'4
        }

        The question is then how to detach just one of the two markups that
        compare equal to each other?

        Passing in one of the markup objects directory doesn't work. This is
        because detach tests for equality to input argument:

        >>> abjad.detach(markup_2, staff[0])
        (Markup(contents=['with the others'], direction=Up), Markup(contents=['with the others'], direction=Up))

        >>> abjad.show(staff) # doctest: +SKIP
        
        >>> abjad.f(staff, strict=50)
        \new Staff
        {
            c'4
            ^ \markup { tutti }                           %! +SCORE
            d'4
            e'4
            f'4
        }

        We start again:

        >>> staff = abjad.Staff("c'4 d' e' f'")
        >>> abjad.attach(markup_1, staff[0], tag='+SCORE')
        >>> abjad.attach(
        ...     markup_2,
        ...     staff[0],
        ...     deactivate=True,
        ...     tag='+PARTS_VIOLIN_1',
        ...     )
        >>> abjad.attach(
        ...     markup_3,
        ...     staff[0],
        ...     deactivate=True,
        ...     tag='+PARTS_VIOLIN_2',
        ...     )
        >>> abjad.show(staff) # doctest: +SKIP

        >>> abjad.f(staff, strict=50)
        \new Staff
        {
            c'4
            ^ \markup { tutti }                           %! +SCORE
        %@% ^ \markup { "with the others" }               %! +PARTS_VIOLIN_1
        %@% ^ \markup { "with the others" }               %! +PARTS_VIOLIN_2
            d'4
            e'4
            f'4
        }

        This time we set ``by_id`` to true. Now detach checks the exact id of
        its input argument (rather than just testing for equality). This gives
        us what we want:

        >>> abjad.detach(markup_2, staff[0], by_id=True)
        (Markup(contents=['with the others'], direction=Up),)

        >>> abjad.show(staff) # doctest: +SKIP
        
        >>> abjad.f(staff, strict=50)
        \new Staff
        {
            c'4
            ^ \markup { tutti }                           %! +SCORE
        %@% ^ \markup { "with the others" }               %! +PARTS_VIOLIN_2
            d'4
            e'4
            f'4
        }

    ..  container:: example

        REGRESSION. Attach-detach-attach pattern works correctly when detaching
        wrappers:

        >>> staff = abjad.Staff("c'4 d' e' f'")
        >>> abjad.attach(abjad.Clef('alto'), staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                \clef "alto"
                c'4
                d'4
                e'4
                f'4
            }

        >>> wrapper = abjad.inspect(staff[0]).wrappers()[0]
        >>> abjad.detach(wrapper, wrapper.component)
        (Wrapper(context='Staff', indicator=Clef('alto'), tag=Tag()),)

        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'4
                d'4
                e'4
                f'4
            }

        >>> abjad.attach(abjad.Clef('tenor'), staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                \clef "tenor"
                c'4
                d'4
                e'4
                f'4
            }

    Returns tuple of zero or more detached items.
    """
    import abjad
    if isinstance(argument, abjad.Spanner):
        if by_id is True:
            raise Exception('ignoring by_id with spanner argument.')
        argument._detach()
        return
    assert target is not None
    after_grace_container = None
    grace_container = None
    spanners = []
    inspector = abjad.inspect(target)
    if isinstance(argument, type):
        if issubclass(argument, abjad.Spanner):
            spanners = inspector.spanners(argument)
        elif issubclass(argument, abjad.AfterGraceContainer):
            after_grace_container = inspector.after_grace_container()
        elif issubclass(argument, abjad.GraceContainer):
            grace_container = inspector.grace_container()
        else:
            assert hasattr(target, '_wrappers')
            result = []
            for wrapper in target._wrappers[:]:
                if isinstance(wrapper, argument):
                    target._wrappers.remove(wrapper)
                    result.append(wrapper)
                elif isinstance(wrapper.indicator, argument):
                    wrapper._detach()
                    result.append(wrapper.indicator)
            result = tuple(result)
            return result
    else:
        if isinstance(argument, abjad.Spanner):
            spanners = inspector.spanners(argument)
        elif isinstance(argument, abjad.AfterGraceContainer):
            after_grace_container = inspector.after_grace_container()
        elif isinstance(argument, abjad.GraceContainer):
            grace_container = inspector.grace_container()
        else:
            assert hasattr(target, '_wrappers')
            result = []
            for wrapper in target._wrappers[:]:
                if wrapper is argument:
                    wrapper._detach()
                    result.append(wrapper)
                elif wrapper.indicator == argument:
                    if by_id is True and id(argument) != id(wrapper.indicator):
                        pass
                    else:
                        wrapper._detach()
                        result.append(wrapper.indicator)
            result = tuple(result)
            return result
    items = []
    items.extend(spanners)
    if after_grace_container is not None:
        items.append(after_grace_container)
    if grace_container is not None:
        items.append(grace_container)
    if by_id is True:
        items = [_ for _ in items if id(item) == id(argument)]
    for item in items:
        item._detach()
    items = tuple(items)
    return items



from abjad.system.AbjadObject import AbjadObject


class AbjadValueObject(AbjadObject):
    """
    Abstract base class for classes which compare equally based on their
    storage format.
    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### SPECIAL METHODS ###

    def __copy__(self, *arguments):
        """
        Copies Abjad value object.

        Returns new Abjad value object.
        """
        import abjad
        return abjad.new(self)

    def __eq__(self, argument):
        """
        Is true when all initialization values of Abjad value object equal
        the initialization values of ``argument``.

        Returns true or false.
        """
        import abjad
        return abjad.TestManager.compare_objects(self, argument)

    def __hash__(self):
        """
        Hashes Abjad value object.

        Returns integer.
        """
        from abjad import system
        hash_values = system.StorageFormatManager(self).get_hash_values()
        try:
            result = hash(hash_values)
        except TypeError:
            raise TypeError(f'unhashable type: {self}')
        return result



import collections
import typing
from abjad import mathtools
from abjad.system.AbjadValueObject import AbjadValueObject
from abjad.mathtools.NonreducedFraction import NonreducedFraction
from abjad.system.FormatSpecification import FormatSpecification
from abjad.system.StorageFormatManager import StorageFormatManager
from abjad.utilities.Duration import Duration
from abjad.utilities.Multiplier import Multiplier


class TimeSignature(AbjadValueObject):
    r"""
    Time signature.

    ..  container:: example

        >>> staff = abjad.Staff("c'8 d'8 e'8")
        >>> time_signature = abjad.TimeSignature((3, 8))
        >>> abjad.attach(time_signature, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                \time 3/8
                c'8
                d'8
                e'8
            }

    ..  container:: example

        Create score-contexted time signatures like this:

        >>> staff = abjad.Staff("c'8 d'8 e'8 c'8 d'8 e'8")
        >>> time_signature = abjad.TimeSignature((3, 8))
        >>> abjad.attach(time_signature, staff[0], context='Score')

        Score-contexted time signatures format behind comments when no Abjad
        score container is found:

        >>> abjad.f(staff)
        \new Staff
        {
            %%% \time 3/8 %%%
            c'8
            d'8
            e'8
            c'8
            d'8
            e'8
        }

        >>> abjad.show(staff) # doctest: +SKIP

        Score-contexted time signatures format normally when an Abjad score
        container is found:

        >>> score = abjad.Score([staff])
        >>> abjad.f(score)
        \new Score
        <<
            \new Staff
            {
                \time 3/8
                c'8
                d'8
                e'8
                c'8
                d'8
                e'8
            }
        >>

        >>> abjad.show(score) # doctest: +SKIP

    ..  container:: example

        Time signatures can be tagged:

        >>> staff = abjad.Staff("c'8 d'8 e'8 c'8 d'8 e'8")
        >>> time_signature = abjad.TimeSignature((3, 8))
        >>> abjad.attach(time_signature, staff[0], context='Score', tag='RED')
        >>> score = abjad.Score([staff])
        >>> abjad.show(staff) # doctest: +SKIP

        >>> abjad.f(score)
        \new Score
        <<
            \new Staff
            {
                \time 3/8 %! RED
                c'8
                d'8
                e'8
                c'8
                d'8
                e'8
            }
        >>

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_denominator',
        '_has_non_power_of_two_denominator',
        '_hide',
        '_multiplier',
        '_numerator',
        '_partial',
        '_partial_repr_string',
        )

    _context = 'Staff'

    _format_slot = 'opening'

    _parameter = True

    ### INITIALIZER ###

    def __init__(
        self,
        pair: typing.Tuple[int, int] = (4, 4),
        *,
        partial: Duration = None,
        hide: bool = None,
        ) -> None:
        import abjad
        pair = getattr(pair, 'pair', pair)
        assert isinstance(pair, collections.Iterable), repr(pair)
        assert len(pair) == 2, repr(pair)
        numerator, denominator = pair
        assert isinstance(numerator, int), repr(numerator)
        assert isinstance(denominator, int), repr(denominator)
        self._numerator: int = numerator
        self._denominator: int = denominator
        if partial is not None:
            partial = Duration(partial)
        self._partial: typing.Optional[Duration] = partial
        if partial is not None:
            self._partial_repr_string = ', partial=%r' % self._partial
        else:
            self._partial_repr_string = ''
        if hide is not None:
            hide = bool(hide)
        self._hide: typing.Optional[bool] = hide
        self._multiplier = self.implied_prolation
        result = mathtools.is_nonnegative_integer_power_of_two(self.denominator)
        assert isinstance(result, bool)
        self._has_non_power_of_two_denominator: bool = not(result)

    ### SPECIAL METHODS ###

    def __add__(self, argument) -> 'TimeSignature':
        """
        Adds time signature to ``argument``.

        ..  container:: example

            Adds two time signatures with the same denominator:

            >>> abjad.TimeSignature((3, 4)) + abjad.TimeSignature((3, 4))
            TimeSignature((6, 4))

        ..  container:: example

            Adds two time signatures with different denominators:

            >>> abjad.TimeSignature((3, 4)) + abjad.TimeSignature((6, 8))
            TimeSignature((12, 8))

            Returns new time signature in terms of greatest denominator.

        """
        if not isinstance(argument, type(self)):
            raise Exception('must be time signature: {argument!r}.')
        nonreduced_1 = NonreducedFraction(
            self.numerator,
            self.denominator,
            )
        nonreduced_2 = NonreducedFraction(
            argument.numerator,
            argument.denominator,
            )
        result = nonreduced_1 + nonreduced_2
        result = type(self)((
            result.numerator,
            result.denominator,
            ))
        return result

    def __copy__(self, *arguments) -> 'TimeSignature':
        """Copies time signature.
        """
        return type(self)(
            (self.numerator, self.denominator),
            partial=self.partial,
            )

    def __eq__(self, argument) -> bool:
        """
        Is true when ``argument`` is a time signature with numerator and
        denominator equal to this time signature. Also true when ``argument``
        is a tuple with first and second elements equal to numerator and
        denominator of this time signature.
        """
        # custom definition retained only bc tests currently break with super()
        if isinstance(argument, type(self)):
            return (self.numerator == argument.numerator and
                self.denominator == argument.denominator)
        elif isinstance(argument, tuple):
            return self.numerator == argument[0] and self.denominator == argument[1]
        else:
            return False

    def __format__(self, format_specification='') -> str:
        """
        Formats time signature.

        ..  container:: example

            >>> print(format(abjad.TimeSignature((3, 8))))
            abjad.TimeSignature((3, 8))

        """
        if format_specification in ('', 'storage'):
            return StorageFormatManager(self).get_storage_format()
        assert format_specification == 'lilypond'
        return self._get_lilypond_format()

    def __ge__(self, argument) -> bool:
        """
        Is true when duration of time signature is greater than or equal to
        duration of ``argument``.
        """
        if isinstance(argument, type(self)):
            return self.duration >= argument.duration
        else:
            raise TypeError(argument)

    def __gt__(self, argument) -> bool:
        """
        Is true when duration of time signature is greater than duration of
        ``argument``.
        """
        if isinstance(argument, type(self)):
            return self.duration > argument.duration
        else:
            raise TypeError(argument)

    def __hash__(self) -> int:
        """Hashes time signature.

        Redefined in tandem with __eq__.
        """
        return super().__hash__()

    def __le__(self, argument) -> bool:
        """
        Is true when duration of time signature is less than duration of
        ``argument``.
        """
        if isinstance(argument, type(self)):
            return self.duration <= argument.duration
        else:
            raise TypeError(argument)

    def __lt__(self, argument) -> bool:
        """
        Is true when duration of time signature is less than duration of
        ``argument``.
        """
        if isinstance(argument, type(self)):
            return self.duration < argument.duration
        else:
            raise TypeError(argument)

    def __radd__(self, argument) -> 'TimeSignature':
        """
        Adds ``argument`` to time signature.

        ..  container:: example

            >>> abjad.TimeSignature((3, 8)) + abjad.TimeSignature((4, 4))
            TimeSignature((11, 8))

        """
        return self.__add__(argument)

    def __str__(self) -> str:
        """
        Gets string representation of time signature.

        ..  container:: example

            >>> str(abjad.TimeSignature((3, 8)))
            '3/8'

        """
        return f'{self.numerator}/{self.denominator}'

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        storage_format_is_indented = False
        if self.partial is not None or self.hide is not None:
            storage_format_is_indented = True
        return FormatSpecification(
            client=self,
            repr_is_indented=False,
            storage_format_args_values=[self.pair],
            storage_format_kwargs_names=['partial', 'hide'],
            storage_format_is_indented=storage_format_is_indented,
            )

    def _get_lilypond_format(self):
        if self.hide:
            return []
        elif self.partial is None:
            return rf'\time {self.numerator}/{self.denominator}'
        else:
            result = []
            duration_string = self.partial.lilypond_duration_string
            partial_directive = rf'\partial {duration_string}'
            result.append(partial_directive)
            string = rf'\time {self.numerator}/{self.denominator}'
            result.append(string)
            return result

    ### PUBLIC PROPERTIES ###

    @property
    def context(self) -> str:
        """
        Gets (historically conventional) context.

        ..  container:: example

            >>> abjad.TimeSignature((3, 8)).context
            'Staff'

        ..  todo:: Should return ``'Score'``.

        Override with ``abjad.attach(..., context='...')``.
        """
        return self._context

    @property
    def denominator(self) -> int:
        """
        Gets denominator of time signature:

        ..  container:: example

            >>> abjad.TimeSignature((3, 8)).denominator
            8

        """
        return self._denominator

    @property
    def duration(self) -> Duration:
        """
        Gets duration of time signature.

        ..  container:: example

            >>> abjad.TimeSignature((3, 8)).duration
            Duration(3, 8)

        """
        return Duration(self.numerator, self.denominator)

    @property
    def has_non_power_of_two_denominator(self) -> bool:
        """
        Is true when time signature has non-power-of-two denominator.

        ..  container:: example

            With non-power-of-two denominator:

            >>> time_signature = abjad.TimeSignature((7, 12))
            >>> time_signature.has_non_power_of_two_denominator
            True

        ..  container:: example

            With power-of-two denominator:

            >>> time_signature = abjad.TimeSignature((3, 8))
            >>> time_signature.has_non_power_of_two_denominator
            False

        """
        return self._has_non_power_of_two_denominator

    @property
    def hide(self) -> typing.Optional[bool]:
        r"""
        Is true when time signature should not appear in output (but should
        still determine effective time signature).

        ..  container:: example

            >>> staff = abjad.Staff("c'4 d' e' f'")
            >>> time_signature = abjad.TimeSignature((4, 4))
            >>> abjad.attach(time_signature, staff[0]) 
            >>> time_signature = abjad.TimeSignature((2, 4), hide=True)
            >>> abjad.attach(time_signature, staff[2]) 
            >>> abjad.show(staff) # doctest: +SKIP

            >>> abjad.f(staff)
            \new Staff
            {
                \time 4/4
                c'4
                d'4
                e'4
                f'4
            }

            >>> for leaf in abjad.iterate(staff).leaves():
            ...     prototype = abjad.TimeSignature 
            ...     leaf, abjad.inspect(leaf).effective(prototype)
            ...
            (Note("c'4"), TimeSignature((4, 4)))
            (Note("d'4"), TimeSignature((4, 4)))
            (Note("e'4"), TimeSignature((2, 4), hide=True))
            (Note("f'4"), TimeSignature((2, 4), hide=True))

        """
        return self._hide

    @property
    def implied_prolation(self) -> Multiplier:
        """
        Gets implied prolation of time signature.

        ..  container:: example

            Implied prolation of time signature with power-of-two denominator:

            >>> abjad.TimeSignature((3, 8)).implied_prolation
            Multiplier(1, 1)

        ..  container:: example

            Implied prolation of time signature with non-power-of-two
            denominator:

            >>> abjad.TimeSignature((7, 12)).implied_prolation
            Multiplier(2, 3)

        """
        dummy_duration = Duration(1, self.denominator)
        return dummy_duration.implied_prolation

    @property
    def numerator(self) -> int:
        """
        Gets numerator of time signature.

        ..  container:: example

            >>> abjad.TimeSignature((3, 8)).numerator
            3

        """
        return self._numerator

    @property
    def pair(self) -> typing.Tuple[int, int]:
        """
        Gets numerator / denominator pair corresponding to time siganture.

        ..  container:: example

            >>> abjad.TimeSignature((3, 8)).pair
            (3, 8)

        """
        return (self.numerator, self.denominator)

    @property
    def partial(self) -> typing.Optional[Duration]:
        """
        Gets duration of pick-up to time signature.

        ..  container:: example

            >>> abjad.TimeSignature((3, 8)).partial is None
            True

        """
        return self._partial

    @property
    def parameter(self) -> bool:
        """
        Is true.

        ..  container:: example

            >>> abjad.TimeSignature((3, 8)).parameter
            True

        """
        return self._parameter

    @property
    def tweaks(self) -> None:
        r"""
        Are not implemented on time signature.
        
        The LilyPond ``\time`` command refuses tweaks.

        Override the LilyPond ``TimeSignature`` grob instead.
        """
        pass

    ### PUBLIC METHODS ###

    @staticmethod
    def from_string(string) -> 'TimeSignature':
        """
        Makes new time signature from fraction ``string``.

        ..  container:: example

            >>> abjad.TimeSignature.from_string('6/8')
            TimeSignature((6, 8))

        """
        assert isinstance(string, str), repr(string)
        parts = string.split('/')
        assert len(parts) == 2, repr(parts)
        numbers = [int(_) for _ in parts]
        numerator, denominator = numbers
        return TimeSignature((numerator, denominator))

    def with_power_of_two_denominator(
        self,
        contents_multiplier=1,
        ) -> 'TimeSignature':
        """
        Makes new time signature equivalent to current time signature with
        power-of-two denominator.

        ..  container:: example

            Non-power-of-two denominator with power-of-two denominator:

            >>> time_signature = abjad.TimeSignature((3, 12))
            >>> time_signature.with_power_of_two_denominator()
            TimeSignature((2, 8))

        """
        contents_multiplier = Multiplier(contents_multiplier)
        contents_multiplier = Multiplier(contents_multiplier)
        non_power_of_two_denominator = self.denominator
        if contents_multiplier == Multiplier(1):
            power_of_two_denominator = \
                mathtools.greatest_power_of_two_less_equal(
                    non_power_of_two_denominator)
        else:
            power_of_two_denominator = \
                mathtools.greatest_power_of_two_less_equal(
                    non_power_of_two_denominator, 1)
        non_power_of_two_pair = NonreducedFraction(self.pair)
        power_of_two_fraction = non_power_of_two_pair.with_denominator(
            power_of_two_denominator)
        power_of_two_pair = power_of_two_fraction.pair
        return type(self)(power_of_two_pair)



#! /usr/bin/env python
import abjad
import copy


class PartCantusScoreTemplate:
    """
    Pärt Cantus score template.
    """

    ### SPECIAL METHODS ###

    def __call__(self):
        """
        Calls score template.

        Returns LilyPond file.
        """
        # make bell voice and staff
        bell_voice = abjad.Voice(name='Bell Voice')
        bell_staff = abjad.Staff([bell_voice], name='Bell Staff')
        # make first violin voice and staff
        first_violin_voice = abjad.Voice(name='First Violin Voice')
        first_violin_staff = abjad.Staff(
            [first_violin_voice],
            name='First Violin Staff',
            )
        # make second violin voice and staff
        second_violin_voice = abjad.Voice(name='Second Violin Voice')
        second_violin_staff = abjad.Staff(
            [second_violin_voice],
            name='Second Violin Staff',
            )
        # make viola voice and staff
        viola_voice = abjad.Voice(name='Viola Voice')
        viola_staff = abjad.Staff([viola_voice], name='Viola Staff')
        # make cello voice and staff
        cello_voice = abjad.Voice(name='Cello Voice')
        cello_staff = abjad.Staff([cello_voice], name='Cello Staff')
        # make bass voice and staff
        bass_voice = abjad.Voice(name='Bass Voice')
        bass_staff = abjad.Staff([bass_voice], name='Bass Staff')
        # make strings staff group
        strings_staff_group = abjad.StaffGroup([
            first_violin_staff,
            second_violin_staff,
            viola_staff,
            cello_staff,
            bass_staff,
            ],
            name='Strings Staff Group',
            )
        # make score
        score = abjad.Score([
            bell_staff,
            strings_staff_group,
            ],
            name='Pärt Cantus Score'
            )
        # return Pärt Cantus score
        return score


def create_pitch_contour_reservoir():
    """
    Creates pitch contour reservoir.
    """
    import abjadext.tonality
    scale = abjadext.tonality.Scale(('a', 'minor'))
    pitch_ranges = {
        'First Violin': abjad.PitchRange('[C4, A6]'),
        'Second Violin': abjad.PitchRange('[A3, A5]'),
        'Viola': abjad.PitchRange('[E3, A4]'),
        'Cello': abjad.PitchRange('[A2, A3]'),
        'Bass': abjad.PitchRange('[C3, A3]'),
    }
    reservoir = {}
    for name, pitch_range in pitch_ranges.items():
        pitch_set = scale.create_named_pitch_set_in_pitch_range(pitch_range)
        pitches = sorted(pitch_set, reverse=True)
        pitch_descents = []
        for i in range(len(pitches)):
            descent = tuple(pitches[:i + 1])
            pitch_descents.append(descent)
        reservoir[name] = tuple(pitch_descents)
    return reservoir


def durate_pitch_contour_reservoir(pitch_contour_reservoir):
    """
    Durates pitch contour reservoir.
    """
    names = [
        'First Violin',
        'Second Violin',
        'Viola',
        'Cello',
        'Bass',
        ]
    durated_reservoir = {}
    for i, name in enumerate(names):
        long_duration = abjad.Duration(1, 2) * pow(2, i)
        short_duration = long_duration / 2
        rest_duration = long_duration * abjad.Multiplier(3, 2)
        div = rest_duration // abjad.Duration(3, 2)
        mod = rest_duration % abjad.Duration(3, 2)
        initial_rest = abjad.MultimeasureRest((3, 2)) * div
        maker = abjad.LeafMaker()
        if mod:
            initial_rest += maker([None], mod)
        durated_contours = [tuple(initial_rest)]
        pitch_contours = pitch_contour_reservoir[name]
        durations = [long_duration, short_duration]
        counter = 0
        maker = abjad.LeafMaker()
        for pitch_contour in pitch_contours:
            contour = []
            for pitch in pitch_contour:
                leaves = maker([pitch], [durations[counter]])
                contour.extend(leaves)
                counter = (counter + 1) % 2
            durated_contours.append(tuple(contour))
        durated_reservoir[name] = tuple(durated_contours)
    return durated_reservoir


def shadow_pitch_contour_reservoir(pitch_contour_reservoir):
    """
    Shadows pitch contour reservoir.
    """
    shadow_pitch_lookup = {
        abjad.NamedPitchClass('a'): -5,  # add a P4 below
        abjad.NamedPitchClass('g'): -3,  # add a m3 below
        abjad.NamedPitchClass('f'): -1,  # add a m2 below
        abjad.NamedPitchClass('e'): -4,  # add a M3 below
        abjad.NamedPitchClass('d'): -2,  # add a M2 below
        abjad.NamedPitchClass('c'): -3,  # add a m3 below
        abjad.NamedPitchClass('b'): -2,  # add a M2 below
    }
    shadowed_reservoir = {}
    for name, pitch_contours in pitch_contour_reservoir.items():
        # The viola does not receive any diads
        if name == 'Viola':
            shadowed_reservoir['Viola'] = pitch_contours
            continue
        shadowed_pitch_contours = []
        for pitch_contour in pitch_contours[:-1]:
            shadowed_pitch_contour = []
            for pitch in pitch_contour:
                pitch_class = pitch.pitch_class
                shadow_pitch = pitch + shadow_pitch_lookup[pitch_class]
                diad = (shadow_pitch, pitch)
                shadowed_pitch_contour.append(diad)
            shadowed_pitch_contours.append(tuple(shadowed_pitch_contour))
        # treat the final contour differently: the last note does not become a diad
        final_shadowed_pitch_contour = []
        for pitch in pitch_contours[-1][:-1]:
            pitch_class = pitch.pitch_class
            shadow_pitch = pitch + shadow_pitch_lookup[pitch_class]
            diad = (shadow_pitch, pitch)
            final_shadowed_pitch_contour.append(diad)
        final_shadowed_pitch_contour.append(pitch_contours[-1][-1])
        shadowed_pitch_contours.append(tuple(final_shadowed_pitch_contour))
        shadowed_reservoir[name] = tuple(shadowed_pitch_contours)
    return shadowed_reservoir


def add_bell_music_to_score(score):
    """
    Adds bell music to score.
    """
    def make_bell_phrase():
        phrase = []
        for _ in range(3):
            measure = abjad.Measure((6, 4), r"r2. a'2.")
            abjad.attach(abjad.LaissezVibrer(), measure[-1])
            phrase.append(measure)
            phrase.append(abjad.Measure((6, 4), 'R1.'))
        for _ in range(2):
            phrase.append(abjad.Measure((6, 4), 'R1.'))
        return phrase
    bell_voice = score['Bell Voice']
    for _ in range(11):
        bell_voice.extend(make_bell_phrase())
    for _ in range(19):
        bell_voice.append(abjad.Measure((6, 4), 'R1.'))
    measure = abjad.Measure((6, 4), r"a'1.")
    abjad.attach(abjad.LaissezVibrer(), measure[-1])
    bell_voice.append(measure)


def add_string_music_to_score(score):
    """
    Adds string music to score.
    """
    # generate some pitch and rhythm information
    pitch_contour_reservoir = create_pitch_contour_reservoir()
    shadowed_contour_reservoir = shadow_pitch_contour_reservoir(
        pitch_contour_reservoir)
    durated_reservoir = durate_pitch_contour_reservoir(
        shadowed_contour_reservoir)
    # add six dotted-whole notes and the durated contours to each string voice
    for name, descents in durated_reservoir.items():
        instrument_voice = score['%s Voice' % name]
        instrument_voice.extend("R1. R1. R1. R1. R1. R1.")
        for descent in descents:
            instrument_voice.extend(descent)
    # apply instrument-specific edits
    edit_first_violin_voice(score, durated_reservoir)
    edit_second_violin_voice(score, durated_reservoir)
    edit_viola_voice(score, durated_reservoir)
    edit_cello_voice(score, durated_reservoir)
    edit_bass_voice(score, durated_reservoir)
    # chop all string parts into 6/4 measures
    strings_staff_group = score['Strings Staff Group']
    with abjad.ForbidUpdate(score):
        for voice in abjad.iterate(strings_staff_group).components(abjad.Voice):
            shards = abjad.mutate(voice[:]).split([(6, 4)], cyclic=True)
            for shard in shards:
                measure = abjad.Measure((6, 4), [])
                abjad.mutate(shard).wrap(measure)


def edit_first_violin_voice(score, durated_reservoir):
    """
    Edits first violin voice.
    """
    voice = score['First Violin Voice']
    descents = durated_reservoir['First Violin']
    last_descent = abjad.Selection(descents[-1])
    copied_descent = abjad.mutate(last_descent).copy()
    voice.extend(copied_descent)
    final_sustain_rhythm = [(6, 4)] * 43 + [(1, 2)]
    maker = abjad.NoteMaker()
    final_sustain_notes = maker(["c'"], final_sustain_rhythm)
    voice.extend(final_sustain_notes)
    tie = abjad.Tie()
    abjad.attach(tie, final_sustain_notes)
    voice.extend('r4 r2.')


def edit_second_violin_voice(score, durated_reservoir):
    """
    Edits second violin voice.
    """
    voice = score['Second Violin Voice']
    descents = durated_reservoir['Second Violin']
    last_descent = abjad.Selection(descents[-1])
    copied_descent = abjad.mutate(last_descent).copy()
    copied_descent = list(copied_descent)
    copied_descent[-1].written_duration = abjad.Duration(1, 1)
    copied_descent.append(abjad.Note('a2'))
    for leaf in copied_descent:
        articulation = abjad.Articulation('accent')
        abjad.attach(articulation, leaf)
        articulation = abjad.Articulation('tenuto')
        abjad.attach(articulation, leaf)
    voice.extend(copied_descent)
    final_sustain = []
    for _ in range(32):
        final_sustain.append(abjad.Note('a1.'))
    final_sustain.append(abjad.Note('a2'))
    final_sustain = abjad.Selection(final_sustain)
    articulation = abjad.Articulation('accent')
    abjad.attach(articulation, final_sustain[0])
    articulation = abjad.Articulation('tenuto')
    abjad.attach(articulation, final_sustain[0])
    voice.extend(final_sustain)
    tie = abjad.Tie()
    abjad.attach(tie, final_sustain)
    voice.extend('r4 r2.')


def edit_viola_voice(score, durated_reservoir):
    """
    Edits viola voice.
    """
    voice = score['Viola Voice']
    descents = durated_reservoir['Viola']
    for leaf in descents[-1]:
        articulation = abjad.Articulation('accent')
        abjad.attach(articulation, leaf)
        articulation = abjad.Articulation('tenuto')
        abjad.attach(articulation, leaf)
    last_descent = abjad.Selection(descents[-1])
    copied_descent = abjad.mutate(last_descent).copy()
    for leaf in copied_descent:
        if leaf.written_duration == abjad.Duration(4, 4):
            leaf.written_duration = abjad.Duration(8, 4)
        else:
            leaf.written_duration = abjad.Duration(4, 4)
    voice.extend(copied_descent)
    bridge = abjad.Note('e1')
    articulation = abjad.Articulation('tenuto')
    abjad.attach(articulation, bridge)
    articulation = abjad.Articulation('accent')
    abjad.attach(articulation, bridge)
    voice.append(bridge)
    final_sustain_rhythm = [(6, 4)] * 21 + [(1, 2)]
    maker = abjad.NoteMaker()
    final_sustain_notes = maker(['e'], final_sustain_rhythm)
    articulation = abjad.Articulation('accent')
    abjad.attach(articulation, final_sustain_notes[0])
    articulation = abjad.Articulation('tenuto')
    abjad.attach(articulation, final_sustain_notes[0])
    voice.extend(final_sustain_notes)
    tie = abjad.Tie()
    abjad.attach(tie, final_sustain_notes)
    voice.extend('r4 r2.')


def edit_cello_voice(score, durated_reservoir):
    """
    Edits cello voice.
    """
    voice = score['Cello Voice']
    descents = durated_reservoir['Cello']
    logical_tie = abjad.inspect(voice[-1]).logical_tie()
    for leaf in logical_tie.leaves:
        parent = abjad.inspect(leaf).parentage().parent
        index = parent.index(leaf)
        parent[index] = abjad.Chord(['e,', 'a,'], leaf.written_duration)
    selection = voice[-len(descents[-1]):]
    unison_descent = abjad.mutate(selection).copy()
    voice.extend(unison_descent)
    for chord in unison_descent:
        index = abjad.inspect(chord).parentage().parent.index(chord)
        parent[index] = abjad.Note(
            chord.written_pitches[1], chord.written_duration)
        articulation = abjad.Articulation('accent')
        abjad.attach(articulation, parent[index])
        articulation = abjad.Articulation('tenuto')
        abjad.attach(articulation, parent[index])
    voice.extend('a,1. ~ a,2')
    voice.extend('b,1 ~ b,1. ~ b,1.')
    voice.extend('a,1. ~ a,1. ~ a,1. ~ a,1. ~ a,1. ~ a,2')
    voice.extend('r4 r2.')


def edit_bass_voice(score, durated_reservoir):
    """
    Edits bass voice.
    """
    voice = score['Bass Voice']
    voice[-3:] = '<e, e>\maxima <d, d>\longa <c, c>\maxima <b,>\longa <a,>\maxima r4 r2.'


def apply_bowing_marks(score):
    """
    Applies bowing marks to score.
    """
    # apply alternating upbow and downbow for first two sounding bars
    # of the first violin
    for measure in score['First Violin Voice'][6:8]:
        for i, chord in enumerate(abjad.iterate(measure).components(abjad.Chord)):
            if i % 2 == 0:
                articulation = abjad.Articulation('downbow')
                abjad.attach(articulation, chord)
            else:
                articulation = abjad.Articulation('upbow')
                abjad.attach(articulation, chord)

    # create and apply rebowing markup
    rebow_markup = abjad.Markup.concat([
        abjad.Markup.musicglyph('scripts.downbow'),
        abjad.Markup.hspace(1),
        abjad.Markup.musicglyph('scripts.upbow'),
        ])
    markup = copy.copy(rebow_markup)
    abjad.attach(markup, score['First Violin Voice'][64][0])
    markup = copy.copy(rebow_markup)
    abjad.attach(markup, score['Second Violin Voice'][75][0])
    markup = copy.copy(rebow_markup)
    abjad.attach(markup, score['Viola Voice'][86][0])


def apply_dynamics(score):
    """
    Applies dynamics to score.
    """

    voice = score['Bell Voice']
    dynamic = abjad.Dynamic('ppp')
    abjad.attach(dynamic, voice[0][1])
    dynamic = abjad.Dynamic('pp')
    abjad.attach(dynamic, voice[8][1])
    dynamic = abjad.Dynamic('p')
    abjad.attach(dynamic, voice[18][1])
    dynamic = abjad.Dynamic('mp')
    abjad.attach(dynamic, voice[26][1])
    dynamic = abjad.Dynamic('mf')
    abjad.attach(dynamic, voice[34][1])
    dynamic = abjad.Dynamic('f')
    abjad.attach(dynamic, voice[42][1])
    dynamic = abjad.Dynamic('ff')
    abjad.attach(dynamic, voice[52][1])
    dynamic = abjad.Dynamic('fff')
    abjad.attach(dynamic, voice[60][1])
    dynamic = abjad.Dynamic('ff')
    abjad.attach(dynamic, voice[68][1])
    dynamic = abjad.Dynamic('f')
    abjad.attach(dynamic, voice[76][1])
    dynamic = abjad.Dynamic('mf')
    abjad.attach(dynamic, voice[84][1])
    dynamic = abjad.Dynamic('pp')
    abjad.attach(dynamic, voice[-1][0])

    voice = score['First Violin Voice']
    dynamic = abjad.Dynamic('ppp')
    abjad.attach(dynamic, voice[6][1])
    dynamic = abjad.Dynamic('pp')
    abjad.attach(dynamic, voice[15][0])
    dynamic = abjad.Dynamic('p')
    abjad.attach(dynamic, voice[22][3])
    dynamic = abjad.Dynamic('mp')
    abjad.attach(dynamic, voice[31][0])
    dynamic = abjad.Dynamic('mf')
    abjad.attach(dynamic, voice[38][3])
    dynamic = abjad.Dynamic('f')
    abjad.attach(dynamic, voice[47][0])
    dynamic = abjad.Dynamic('ff')
    abjad.attach(dynamic, voice[55][2])
    dynamic = abjad.Dynamic('fff')
    abjad.attach(dynamic, voice[62][2])

    voice = score['Second Violin Voice']
    dynamic = abjad.Dynamic('pp')
    abjad.attach(dynamic, voice[7][0])
    dynamic = abjad.Dynamic('p')
    abjad.attach(dynamic, voice[12][0])
    dynamic = abjad.Dynamic('p')
    abjad.attach(dynamic, voice[16][0])
    dynamic = abjad.Dynamic('mp')
    abjad.attach(dynamic, voice[25][1])
    dynamic = abjad.Dynamic('mf')
    abjad.attach(dynamic, voice[34][1])
    dynamic = abjad.Dynamic('f')
    abjad.attach(dynamic, voice[44][1])
    dynamic = abjad.Dynamic('ff')
    abjad.attach(dynamic, voice[54][0])
    dynamic = abjad.Dynamic('fff')
    abjad.attach(dynamic, voice[62][1])

    voice = score['Viola Voice']
    dynamic = abjad.Dynamic('p')
    abjad.attach(dynamic, voice[8][0])
    dynamic = abjad.Dynamic('mp')
    abjad.attach(dynamic, voice[19][1])
    dynamic = abjad.Dynamic('mf')
    abjad.attach(dynamic, voice[30][0])
    dynamic = abjad.Dynamic('f')
    abjad.attach(dynamic, voice[36][0])
    dynamic = abjad.Dynamic('f')
    abjad.attach(dynamic, voice[42][0])
    dynamic = abjad.Dynamic('ff')
    abjad.attach(dynamic, voice[52][0])
    dynamic = abjad.Dynamic('fff')
    abjad.attach(dynamic, voice[62][0])

    voice = score['Cello Voice']
    dynamic = abjad.Dynamic('p')
    abjad.attach(dynamic, voice[10][0])
    dynamic = abjad.Dynamic('mp')
    abjad.attach(dynamic, voice[21][0])
    dynamic = abjad.Dynamic('mf')
    abjad.attach(dynamic, voice[31][0])
    dynamic = abjad.Dynamic('f')
    abjad.attach(dynamic, voice[43][0])
    dynamic = abjad.Dynamic('ff')
    abjad.attach(dynamic, voice[52][1])
    dynamic = abjad.Dynamic('fff')
    abjad.attach(dynamic, voice[62][0])

    voice = score['Bass Voice']
    dynamic = abjad.Dynamic('mp')
    abjad.attach(dynamic, voice[14][0])
    dynamic = abjad.Dynamic('mf')
    abjad.attach(dynamic, voice[27][0])
    dynamic = abjad.Dynamic('f')
    abjad.attach(dynamic, voice[39][0])
    dynamic = abjad.Dynamic('ff')
    abjad.attach(dynamic, voice[51][0])
    dynamic = abjad.Dynamic('fff')
    abjad.attach(dynamic, voice[62][0])


def apply_expressive_marks(score):
    """
    Applies expressive marks to score.
    """
    voice = score['First Violin Voice']
    markup = abjad.Markup(
        r'\left-column { div. \line { con sord. } }', direction=abjad.Up)
    abjad.attach(markup, voice[6][1])
    markup = abjad.Markup('sim.', direction=abjad.Up)
    abjad.attach(markup, voice[8][0])
    markup = abjad.Markup('uniti', direction=abjad.Up)
    abjad.attach(markup, voice[58][3])
    markup = abjad.Markup('div.', direction=abjad.Up)
    abjad.attach(markup, voice[59][0])
    markup = abjad.Markup('uniti', direction=abjad.Up)
    abjad.attach(markup, voice[63][3])

    voice = score['Second Violin Voice']
    markup = abjad.Markup('div.', direction=abjad.Up)
    abjad.attach(markup, voice[7][0])
    markup = abjad.Markup('uniti', direction=abjad.Up)
    abjad.attach(markup, voice[66][1])
    markup = abjad.Markup('div.', direction=abjad.Up)
    abjad.attach(markup, voice[67][0])
    markup = abjad.Markup('uniti', direction=abjad.Up)
    abjad.attach(markup, voice[74][0])

    voice = score['Viola Voice']
    markup = abjad.Markup('sole', direction=abjad.Up)
    abjad.attach(markup, voice[8][0])

    voice = score['Cello Voice']
    markup = abjad.Markup('div.', direction=abjad.Up)
    abjad.attach(markup, voice[10][0])
    markup = abjad.Markup('uniti', direction=abjad.Up)
    abjad.attach(markup, voice[74][0])
    markup = abjad.Markup('uniti', direction=abjad.Up)
    abjad.attach(markup, voice[84][1])
    markup = abjad.Markup(r'\italic { espr. }', direction=abjad.Down)
    abjad.attach(markup, voice[86][0])
    markup = abjad.Markup(r'\italic { molto espr. }', direction=abjad.Down)
    abjad.attach(markup, voice[88][1])

    voice = score['Bass Voice']
    markup = abjad.Markup('div.', direction=abjad.Up)
    abjad.attach(markup, voice[14][0])
    markup = abjad.Markup(r'\italic { espr. }', direction=abjad.Down)
    abjad.attach(markup, voice[86][0])
    abjad.mutate(voice[88][:]).split(
        [abjad.Duration(1, 1), abjad.Duration(1, 2)]
        )
    markup = abjad.Markup(r'\italic { molto espr. }', direction=abjad.Down)
    abjad.attach(markup, voice[88][1])
    markup = abjad.Markup('uniti', direction=abjad.Up)
    abjad.attach(markup, voice[99][1])

    strings_staff_group = score['Strings Staff Group']
    for voice in abjad.iterate(strings_staff_group).components(abjad.Voice):
        markup = abjad.Markup(r'\italic { (non dim.) }', direction=abjad.Down)
        abjad.attach(markup, voice[102][0])


def apply_final_bar_lines(score):
    """
    Applies final bar lines to score.
    """
    for voice in abjad.iterate(score).components(abjad.Voice):
        bar_line = abjad.BarLine('|.')
        leaf = abjad.inspect(voice).leaf(-1)
        abjad.attach(bar_line, leaf)


def apply_page_breaks(score):
    """
    Applies page breaks to score.
    """
    bell_voice = score['Bell Voice']
    measure_indices = [
        5, 10, 15, 20,
        25, 30, 35, 40,
        45, 50, 55, 60,
        65, 72, 79, 86,
        93, 100,
        ]
    for measure_index in measure_indices:
        command = abjad.LilyPondLiteral(r'\break', 'after')
        abjad.attach(command, bell_voice[measure_index])


def apply_rehearsal_marks(score):
    """
    Applies rehearsal marks to score.
    """
    bell_voice = score['Bell Voice']
    measure_indices = [
        6, 12, 18, 24,
        30, 36, 42, 48,
        54, 60, 66, 72,
        78, 84, 90, 96,
        102,
        ]
    for measure_index in measure_indices:
        command = abjad.LilyPondLiteral(r'\mark \default', 'before')
        abjad.attach(command, bell_voice[measure_index])


def configure_score(score):
    """
    Configures score.
    """
    # configure bell staff
    bell_staff = score['Bell Staff']
    leaf = abjad.inspect(bell_staff).leaf(0)
    clef = abjad.Clef('treble')
    abjad.attach(clef, leaf)
    bells = abjad.Instrument(
        name='Campana in La',
        short_name='Camp.',
        pitch_range='[C4, C6]',
        )
    abjad.attach(bells, leaf)
    mark = abjad.MetronomeMark((1, 4), (112, 120))
    abjad.attach(mark, leaf)
    time_signature = abjad.TimeSignature((6, 4))
    abjad.attach(time_signature, leaf)
    # configure first violin staff
    first_violin_staff = score['First Violin Staff']
    leaf = abjad.inspect(first_violin_staff).leaf(0)
    clef = abjad.Clef('treble')
    abjad.attach(clef, leaf)
    violin = abjad.Violin(
        markup=abjad.Markup('Violin I'),
        short_markup=abjad.Markup('Vl. I'),
        )
    abjad.attach(violin, leaf)
    # configure second violin staff
    second_violin_staff = score['Second Violin Staff']
    leaf = abjad.inspect(second_violin_staff).leaf(0)
    clef = abjad.Clef('treble')
    abjad.attach(clef, leaf)
    violin = abjad.Violin(
        markup=abjad.Markup('Violin II'),
        short_markup=abjad.Markup('Vl. II'),
        )
    abjad.attach(violin, leaf)
    # configure viola staff
    leaf = abjad.inspect(score['Viola Staff']).leaf(0)
    clef = abjad.Clef('alto')
    abjad.attach(clef, leaf)
    viola = abjad.Viola()
    abjad.attach(viola, leaf)
    # configure cello staff
    leaf = abjad.inspect(score['Cello Staff']).leaf(0)
    clef = abjad.Clef('bass')
    abjad.attach(clef, leaf)
    cello = abjad.Cello()
    abjad.attach(cello, leaf)
    # configure bass staff
    leaf = abjad.inspect(score['Bass Staff']).leaf(0)
    clef = abjad.Clef('bass')
    abjad.attach(clef, leaf)
    contrabass = abjad.Contrabass(
        short_markup=abjad.Markup('Cb.'),
        )
    abjad.attach(contrabass, leaf)
    # configure score
    vector = abjad.SpacingVector(0, 0, 8, 0)
    abjad.override(score).vertical_axis_group.staff_staff_spacing = vector
    abjad.override(score).staff_grouper.staff_staff_spacing = vector
    abjad.override(score).staff_symbol.thickness = 0.5
    scheme = abjad.Scheme('format-mark-box-numbers')
    abjad.setting(score).mark_formatter = scheme


def configure_lilypond_file(lilypond_file):
    """
    Configures LilyPond file.
    """
    lilypond_file._global_staff_size = 8
    context_block = abjad.ContextBlock(
        source_lilypond_type=r'Staff \RemoveEmptyStaves',
        )
    abjad.override(context_block).vertical_axis_group.remove_first = True
    lilypond_file.layout_block.items.append(context_block)
    lilypond_file.paper_block.system_separator_markup = 'slashSeparator'
    bottom_margin = abjad.LilyPondDimension(0.5, 'in')
    lilypond_file.paper_block.bottom_margin = bottom_margin
    top_margin = abjad.LilyPondDimension(0.5, 'in')
    lilypond_file.paper_block.top_margin = top_margin
    left_margin = abjad.LilyPondDimension(0.75, 'in')
    lilypond_file.paper_block.left_margin = left_margin
    right_margin = abjad.LilyPondDimension(0.5, 'in')
    lilypond_file.paper_block.right_margin = right_margin
    paper_width = abjad.LilyPondDimension(5.25, 'in')
    lilypond_file.paper_block.paper_width = paper_width
    paper_height = abjad.LilyPondDimension(7.25, 'in')
    lilypond_file.paper_block.paper_height = paper_height
    lilypond_file.header_block.composer = abjad.Markup('Arvo Pärt')
    title = 'Cantus in Memory of Benjamin Britten (1980)'
    lilypond_file.header_block.title = abjad.Markup(title)


def make_part_lilypond_file():
    """
    Makes Pärt LilyPond file.
    """
    score_template = PartCantusScoreTemplate()
    score = score_template()
    add_bell_music_to_score(score)
    add_string_music_to_score(score)
    apply_bowing_marks(score)
    apply_dynamics(score)
    apply_expressive_marks(score)
    apply_page_breaks(score)
    apply_rehearsal_marks(score)
    apply_final_bar_lines(score)
    configure_score(score)
    lilypond_file = abjad.LilyPondFile.new(score)
    configure_lilypond_file(lilypond_file)
    return lilypond_file


if __name__ == '__main__':
    from abjad import show
    lilypond_file = make_part_lilypond_file()
    show(lilypond_file)



from abjad.system.AbjadValueObject import AbjadValueObject


class LilyPondLanguageToken(AbjadValueObject):
    r"""
    A LilyPond file ``\language`` token.

    ..  container:: example

        >>> abjad.LilyPondLanguageToken()
        LilyPondLanguageToken()

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r"""
        Formats LilyPond language token.

        ..  container:: example

            >>> token = abjad.LilyPondLanguageToken()
            >>> print(format(token))
            \language "english"

        Returns string.
        """
        import abjad
        if format_specification in ('', 'lilypond'):
            return self._get_lilypond_format()
        elif format_specification == 'storage':
            return abjad.StorageFormatManager(self).get_storage_format()
        return str(self)

    def __repr__(self):
        """
        Gets interpreter representation of LilyPond language token.

        ..  container:: example

            >>> token = abjad.LilyPondLanguageToken()
            >>> token
            LilyPondLanguageToken()

        Returns string.
        """
        return '{}()'.format(type(self).__name__)

    ### PRIVATE METHODS ###

    def _get_lilypond_format(self):
        string = r'\language "english"'
        return string



import abc
import operator
from abjad.system.AbjadValueObject import AbjadValueObject


class Inequality(AbjadValueObject):
    """
    Inequality.
    """

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Inequalities'

    __slots__ = (
        '_operator_string',
        '_operator_function',
        )

    _operator_strings = (
        '!=',
        '<',
        '<=',
        '==',
        '>',
        '>=',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        operator_string='<',
        ):
        assert operator_string in self._operator_strings
        self._operator_string = operator_string
        self._operator_function = {
            '!=': operator.ne,
            '<': operator.lt,
            '<=': operator.le,
            '==': operator.eq,
            '>': operator.gt,
            '>=': operator.ge,
            }[self._operator_string]

    ### SPECIAL METHODS ###

    @abc.abstractmethod
    def __call__(self, argument):
        """
        Calls inequality on ``argument``.

        Returns true or false.
        """
        raise NotImplementedError

    ### PUBLIC PROPERTIES ###

    @property
    def operator_string(self):
        """
        Gets operator string.

        Returns string.
        """
        return self._operator_string



from .Vector import Vector


class PitchVector(Vector):
    """
    Pitch vector.

    ..  container:: example

        >>> vector = abjad.PitchVector(
        ...     items=[7, 6, -2, -3, -3, 0, 1, 14, 15, 16, 16],
        ...     item_class=abjad.NumberedPitch,
        ...     )

        >>> items = list(vector.items())
        >>> items.sort(key=lambda x: x[0].number)
        >>> for pitch_class, count in items:
        ...     print(pitch_class, count)
        -3 2
        -2 1
        0 1
        1 1
        6 1
        7 1
        14 1
        15 1
        16 2

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### SPECIAL METHODS ###

    def __repr__(self):
        """
        Gets interpreter representation of pitch vector.

        ..  container:: example

            Gets interpreter representation of pitch vector:

            >>> vector = abjad.PitchVector(
            ...     items=[7, 6, -2, -3, -3, 0, 1, 14, 15, 16, 16],
            ...     item_class=abjad.NumberedPitch,
            ...     )

            >>> vector
            PitchVector({-3: 2, -2: 1, 0: 1, 1: 1, 6: 1, 7: 1, 14: 1, 15: 1, 16: 2}, item_class=NumberedPitch)

        ..  container:: example

            Initializes from interpreter representation of pitch vector:

                >>> abjad.PitchVector(vector)
                PitchVector({-3: 2, -2: 1, 0: 1, 1: 1, 6: 1, 7: 1, 14: 1, 15: 1, 16: 2}, item_class=NumberedPitch)

        Returns string.
        """
        return super().__repr__()

    ### PRIVATE PROPERTIES ###

    @property
    def _named_item_class(self):
        import abjad
        return abjad.NamedPitch

    @property
    def _numbered_item_class(self):
        import abjad
        return abjad.NumberedPitch

    @property
    def _parent_item_class(self):
        import abjad
        return abjad.Pitch

    ### PUBLIC METHODS ###

    @classmethod
    def from_selection(
        class_,
        selection,
        item_class=None,
        ):
        """
        Makes pitch vector from `selection`.

        Returns pitch vector.
        """
        import abjad
        pitch_segment = abjad.PitchSegment.from_selection(selection)
        return class_(
            pitch_segment,
            item_class=item_class,
            )



import os
import subprocess
import time
from .ContextManager import ContextManager


class Timer(ContextManager):
    """
    A timing context manager.

    ..  container:: example

        >>> timer = abjad.Timer()
        >>> with timer:
        ...     for _ in range(1000000):
        ...         x = 1 + 1
        ...
        >>> timer.elapsed_time # doctest: +SKIP
        0.092742919921875

        The timer can also be accessed from within the ``with`` block:

        >>> with abjad.Timer() as timer: # doctest: +SKIP
        ...     for _ in range(5):
        ...         for _ in range(1000000):
        ...             x = 1 + 1
        ...         print(timer.elapsed_time)
        ...
        0.101150989532
        0.203935861588
        0.304930925369
        0.4057970047
        0.50649189949

    Timers can be reused between ``with`` blocks. They will reset their clock
    on entering any ``with`` block.
    """

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Context managers'

    __slots__ = (
        '_enter_message',
        '_exit_message',
        '_print_continuously_from_background',
        '_process',
        '_start_time',
        '_stop_time',
        '_timer_process',
        '_verbose',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        exit_message=None,
        enter_message=None,
        print_continuously_from_background=False,
        verbose=True,
        ):
        if enter_message is not None:
            enter_message = str(enter_message)
        self._enter_message = enter_message
        if exit_message is not None:
            exit_message = str(exit_message)
        self._exit_message = exit_message
        self._print_continuously_from_background = \
            print_continuously_from_background
        self._process = None
        self._timer_process = None
        self._start_time = None
        self._stop_time = None
        self._verbose = bool(verbose)

    ### SPECIAL METHODS ###

    def __enter__(self):
        """
        Enters context manager.

        Returns context manager.
        """
        if self.enter_message and self.verbose:
            print(self.enter_message)
        self._stop_time = None
        self._start_time = time.time()
        if self.print_continuously_from_background:
            from abjad import abjad_configuration
            path = os.path.join(
                abjad_configuration.abjad_directory,
                'scr',
                'timer.py',
                )
            interval = str(int(self.print_continuously_from_background))
            process = subprocess.Popen([path, interval], shell=False)
            self._process = process
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        """
        Exist context manager.

        Returns none.
        """
        self._stop_time = time.time()
        if self._process is not None:
            self._process.kill()
        if self.exit_message and self.verbose:
            print(self.exit_message, self.elapsed_time)

    ### PUBLIC PROPERTIES ###

    @property
    def elapsed_time(self):
        """
        Elapsed time.

        Return float or none.
        """
        if self.start_time is not None:
            if self.stop_time is not None:
                return self.stop_time - self.start_time
            return time.time() - self.start_time
        return None

    @property
    def enter_message(self):
        """
        Timer enter message.

        Returns string.
        """
        return self._enter_message

    @property
    def exit_message(self):
        """
        Timer exit message.

        Returns string.
        """
        return self._exit_message

    @property
    def print_continuously_from_background(self):
        """
        Is true when timer should print continuously from background.

        Returns true or false.
        """
        return self._print_continuously_from_background

    @property
    def start_time(self):
        """
        Start time of timer.

        Returns time.
        """
        return self._start_time

    @property
    def stop_time(self):
        """
        Stop time of timer.

        Returns time.
        """
        return self._stop_time

    @property
    def total_time_message(self):
        """
        Gets total time message.

        Truncated to the nearest second.

        Returns string.
        """
        import abjad
        identifier = abjad.String('second').pluralize(int(self.elapsed_time))
        message = 'total time {} {} ...'
        message = message.format(int(self.elapsed_time), identifier)
        return message

    @property
    def verbose(self):
        """
        Is true if timer should print messages.

        Returns true or false.
        """
        return self._verbose



"""
Abjad's scrape of LilyPond's execution environment.
"""

from abjad.ly.contexts import contexts
from abjad.ly.current_module import current_module
from abjad.ly.drums import drums
from abjad.ly.engravers import engravers
from abjad.ly.grob_interfaces import grob_interfaces
from abjad.ly.interface_properties import interface_properties
from abjad.ly.language_pitch_names import language_pitch_names
from abjad.ly.markup_functions import markup_functions, markup_list_functions
from abjad.ly.music_glyphs import music_glyphs



import abjad


class ScoreTemplate(abjad.AbjadObject):
    """
    Score template.
    """

    def __call__(self):
        """
        Calls score template.

        Returns score.
        """
        voice = abjad.Voice(name='Example Voice')
        staff = abjad.Staff([voice], name='Example Staff')
        score = abjad.Score([staff], name='Example Score')
        return score



import collections
from abjad import mathtools
from abjad.system.AbjadObject import AbjadObject


class Parentage(AbjadObject, collections.Sequence):
    r'''
    Parentage of a component.

    ..  container:: example

        >>> score = abjad.Score()
        >>> string = r"""\new Voice = "Treble Voice" { e'4 }"""
        >>> treble_staff = abjad.Staff(string, name='Treble Staff')
        >>> score.append(treble_staff)
        >>> string = r"""\new Voice = "Bass Voice" { c4 }"""
        >>> bass_staff = abjad.Staff(string, name='Bass Staff')
        >>> clef = abjad.Clef('bass')
        >>> abjad.attach(clef, bass_staff[0][0])
        >>> score.append(bass_staff)
        >>> abjad.show(score) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(score)
            \new Score
            <<
                \context Staff = "Treble Staff"
                {
                    \context Voice = "Treble Voice"
                    {
                        e'4
                    }
                }
                \context Staff = "Bass Staff"
                {
                    \context Voice = "Bass Voice"
                    {
                        \clef "bass"
                        c4
                    }
                }
            >>

        >>> bass_voice = score['Bass Voice']
        >>> note = bass_voice[0]
        >>> for component in abjad.inspect(note).parentage():
        ...     component
        ...
        Note('c4')
        Voice('c4', name='Bass Voice')
        <Staff-"Bass Staff"{1}>
        <Score<<2>>>

    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Selections'

    __slots__ = (
        '_component',
        '_components',
        '_root',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        component=None,
        include_self=True,
        grace_notes=False,
        ):
        import abjad
        assert isinstance(component, (abjad.Component, type(None)))
        self._component = component
        if component is None:
            components = ()
        else:
            components = []
            if include_self:
                parent = component
            else:
                parent = component._parent
            prototype = (abjad.AfterGraceContainer, abjad.GraceContainer)
            while parent is not None:
                components.append(parent)
                if grace_notes and isinstance(parent, prototype):
                    parent = parent._carrier
                else:
                    parent = parent._parent
            components = tuple(components)
        self._components = components

    ### SPECIAL METHODS ###

    def __getitem__(self, argument):
        """
        Gets ``argument``.

        Returns component or tuple of components.
        """
        return self.components.__getitem__(argument)

    def __len__(self):
        """
        Gets number of components in parentage.

        Returns nonnegative integer.
        """
        return len(self.components)

    ### PRIVATE PROPERTIES ###

    @property
    def _prolations(self):
        import abjad
        prolations = []
        default = abjad.Multiplier(1)
        for parent in self:
            prolation = getattr(parent, 'implied_prolation', default)
            prolations.append(prolation)
        return prolations

    ### PRIVATE METHODS ###

    def _get_governor(self):
        import abjad
        for component in self:
            if (isinstance(component, abjad.Container) and
                not component.is_simultaneous):
                if component._parent is None:
                    return component
                if (isinstance(component._parent, abjad.Container) and
                    component._parent.is_simultaneous):
                    return component

    @staticmethod
    def _id_string(component):
        lhs = component.__class__.__name__
        rhs = getattr(component, 'name', None) or id(component)
        return f'{lhs}-{rhs!r}'

    ### PUBLIC PROPERTIES ###

    @property
    def component(self):
        """
        The component from which the selection was derived.

        Returns component.
        """
        return self._component

    @property
    def components(self):
        """
        Gets components.

        Returns tuple.
        """
        return self._components

    @property
    def depth(self):
        """
        Length of proper parentage of component.

        Returns nonnegative integer.
        """
        return len(self[1:])

    @property
    def is_grace_note(self):
        r"""
        Is true when parentage contains a grace container.

        .. container:: example

            Grace notes:

            >>> voice = abjad.Voice("c'4 d'4 e'4 f'4")
            >>> container = abjad.GraceContainer("c'16 d'16")
            >>> abjad.attach(container, voice[1])
            >>> abjad.show(voice) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(voice)
                \new Voice
                {
                    c'4
                    \grace {
                        c'16
                        d'16
                    }
                    d'4
                    e'4
                    f'4
                }

            >>> for leaf in abjad.iterate(voice).leaves():
            ...     parentage = abjad.inspect(leaf).parentage()
            ...     print(leaf, parentage.is_grace_note)
            ...
            c'4 False
            c'16 True
            d'16 True
            d'4 False
            e'4 False
            f'4 False

        Returns true or false.
        """
        import abjad
        grace_container = self.get_first(prototype=abjad.GraceContainer)
        if grace_container is not None:
            return True
        return False

    @property
    def is_orphan(self):
        """
        Is true when component has no parent.

        Returns true or false.
        """
        return self.parent is None

    @property
    def logical_voice(self):
        r"""
        Gets logical voice.

        ..  container:: example

            Gets logical voice of note:

            >>> voice = abjad.Voice("c'4 d'4 e'4 f'4", name='CustomVoice')
            >>> staff = abjad.Staff([voice], name='CustomStaff')
            >>> score = abjad.Score([staff], name='CustomScore')
            >>> abjad.show(score) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(score)
                \context Score = "CustomScore"
                <<
                    \context Staff = "CustomStaff"
                    {
                        \context Voice = "CustomVoice"
                        {
                            c'4
                            d'4
                            e'4
                            f'4
                        }
                    }
                >>

            >>> note = voice[0]
            >>> parentage = abjad.inspect(note).parentage()
            >>> logical_voice = parentage.logical_voice

            >>> for key, value in logical_voice.items():
            ...     print('%12s: %s' % (key, value))
            ...
            score: Score-'CustomScore'
            staff group:
            staff: Staff-'CustomStaff'
            voice: Voice-'CustomVoice'

        Returns ordered dictionary.
        """
        import abjad
        keys = ('score', 'staff group', 'staff', 'voice')
        logical_voice = collections.OrderedDict.fromkeys(keys, '')
        for component in self:
            if isinstance(component, abjad.Voice):
                if not logical_voice['voice']:
                    logical_voice['voice'] = self._id_string(component)
            elif isinstance(component, abjad.Staff):
                if not logical_voice['staff']:
                    logical_voice['staff'] = self._id_string(component)
                    # explicit staff demands a nested voice:
                    # if no explicit voice has been found,
                    # create implicit voice here with random integer
                    if not logical_voice['voice']:
                        logical_voice['voice'] = id(component)
            elif isinstance(component, abjad.StaffGroup):
                if not logical_voice['staff group']:
                    logical_voice['staff group'] = self._id_string(component)
            elif isinstance(component, abjad.Score):
                if not logical_voice['score']:
                    logical_voice['score'] = self._id_string(component)
        return logical_voice

    @property
    def parent(self):
        """
        Gets parent.

        Returns none when component has no parent.

        Returns component or none.
        """
        if 1 < len(self):
            return self[1]

    @property
    def prolation(self):
        """
        Gets prolation.

        Returns multiplier.
        """
        import abjad
        prolations = [abjad.Multiplier(1)] + self._prolations
        products = mathtools.cumulative_products(prolations)
        return products[-1]

    @property
    def root(self):
        """
        Gets root.

        Root defined equal to last component in parentage.

        Returns component.
        """
        return self[-1]

    @property
    def score_index(self):
        r"""
        Gets score index.

        ..  todo:: Define score index for grace notes.

        ..  container:: example

            Gets note score indices:

            >>> staff_1 = abjad.Staff(r"\times 2/3 { c''2 b'2 a'2 }")
            >>> staff_2 = abjad.Staff("c'2 d'2")
            >>> score = abjad.Score([staff_1, staff_2])
            >>> abjad.show(score) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(score)
                \new Score
                <<
                    \new Staff
                    {
                        \times 2/3 {
                            c''2
                            b'2
                            a'2
                        }
                    }
                    \new Staff
                    {
                        c'2
                        d'2
                    }
                >>

            >>> for leaf in abjad.select(score).leaves():
            ...     parentage = abjad.inspect(leaf).parentage()
            ...     leaf, parentage.score_index
            ...
            (Note("c''2"), (0, 0, 0))
            (Note("b'2"), (0, 0, 1))
            (Note("a'2"), (0, 0, 2))
            (Note("c'2"), (1, 0))
            (Note("d'2"), (1, 1))

        ..  container:: example

            With grace notes:

            >>> voice = abjad.Voice("c'8 [ d'8 e'8 f'8 ]")
            >>> container = abjad.GraceContainer("cf''16 bf'16")
            >>> abjad.attach(container, voice[1])
            >>> abjad.show(voice) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(voice)
                \new Voice
                {
                    c'8
                    [
                    \grace {
                        cf''16
                        bf'16
                    }
                    d'8
                    e'8
                    f'8
                    ]
                }

            >>> leaves = abjad.iterate(voice).components()
            >>> for leaf in leaves:
            ...     parentage = abjad.inspect(leaf).parentage()
            ...     leaf, parentage.score_index
            ...
            (Voice("c'8 d'8 e'8 f'8"), ())
            (Note("c'8"), (0,))
            (Note("cf''16"), (0,))
            (Note("bf'16"), (1,))
            (Note("d'8"), (1,))
            (Note("e'8"), (2,))
            (Note("f'8"), (3,))

            ..  todo:: Incorrect values returned for grace notes.

        Returns tuple of zero or more nonnegative integers.
        """
        result = []
        current = self[0]
        for parent in self[1:]:
            index = parent.index(current)
            result.insert(0, index)
            current = parent
        result = tuple(result)
        return result

    @property
    def tuplet_depth(self):
        r"""
        Gets tuplet depth.

        ..  container:: example

            Gets tuplet depth:

            >>> tuplet = abjad.Tuplet((2, 3), "c'2 d'2 e'2")
            >>> staff = abjad.Staff([tuplet])
            >>> note = tuplet[0]
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                {
                    \times 2/3 {
                        c'2
                        d'2
                        e'2
                    }
                }

            >>> abjad.inspect(note).parentage().tuplet_depth
            1

            >>> abjad.inspect(tuplet).parentage().tuplet_depth
            0

            >>> abjad.inspect(staff).parentage().tuplet_depth
            0

        Returns nonnegative integer.
        """
        import abjad
        result = 0
        # should probably interate up to only first simultaneous container
        # in parentage.
        # note that we probably need a named idea for 'parentage
        # up to first simultaneous container'.
        for parent in self[1:]:
            if isinstance(parent, abjad.Tuplet):
                result += 1
        return result

    ### PUBLIC METHODS ###

    def get_first(self, prototype=None):
        """
        Gets first instance of ``prototype`` in parentage.

        Returns component or none.
        """
        import abjad
        if prototype is None:
            prototype = (abjad.Component,)
        if not isinstance(prototype, tuple):
            prototype = (prototype,)
        for component in self:
            if isinstance(component, prototype):
                return component



def greatest_common_divisor(*integers):
    """
    Calculates greatest common divisor of ``integers``.

    ..  container:: example

        >>> abjad.mathtools.greatest_common_divisor(84, -94, -144)
        2

    Allows nonpositive input.

    Raises not implemented error when zero is included in input.

    Returns positive integer.
    """
    from abjad import mathtools
    common_divisors = None
    for positive_integer in integers:
        all_divisors = set(mathtools.divisors(positive_integer))
        if common_divisors is None:
            common_divisors = all_divisors
        else:
            common_divisors &= all_divisors
            if common_divisors == set([1]):
                return 1
    return max(common_divisors)



import typing
from abjad.system.AbjadValueObject import AbjadValueObject
from .LilyPondEngraver import LilyPondEngraver
from .LilyPondGrob import LilyPondGrob


class LilyPondContext(AbjadValueObject):
    r"""
    LilyPond context.

    ..  container:: example

        >>> context = abjad.LilyPondContext('MensuralStaff')
        >>> abjad.f(context)
        abjad.LilyPondContext(
            name='MensuralStaff',
            )

        >>> for lilypond_context in abjad.LilyPondContext.list_all_contexts():
        ...     is_global_context = 'X' if lilypond_context.is_global_context else ' '
        ...     is_score_context = 'X' if lilypond_context.is_score_context else ' '
        ...     is_staff_group_context = 'X' if lilypond_context.is_staff_group_context else ' '
        ...     is_staff_context = 'X' if lilypond_context.is_staff_context else ' '
        ...     is_bottom_context = 'X' if lilypond_context.is_bottom_context else ' '
        ...     print('[{}] [{}] [{}] [{}] [{}] {}'.format(
        ...         is_global_context,
        ...         is_score_context,
        ...         is_staff_group_context,
        ...         is_staff_context,
        ...         is_bottom_context,
        ...         lilypond_context.name,
        ...         ))
        ...
        [ ] [ ] [X] [ ] [ ] ChoirStaff
        [ ] [ ] [ ] [ ] [X] ChordNames
        [ ] [ ] [ ] [ ] [X] CueVoice
        [ ] [ ] [ ] [ ] [X] Devnull
        [ ] [ ] [ ] [X] [ ] DrumStaff
        [ ] [ ] [ ] [ ] [X] DrumVoice
        [ ] [ ] [ ] [ ] [X] Dynamics
        [ ] [ ] [ ] [ ] [X] FiguredBass
        [ ] [ ] [ ] [ ] [X] FretBoards
        [X] [ ] [ ] [ ] [ ] Global
        [ ] [ ] [X] [ ] [ ] GrandStaff
        [ ] [ ] [ ] [X] [ ] GregorianTranscriptionStaff
        [ ] [ ] [ ] [ ] [X] GregorianTranscriptionVoice
        [ ] [ ] [ ] [X] [ ] KievanStaff
        [ ] [ ] [ ] [ ] [X] KievanVoice
        [ ] [ ] [ ] [ ] [X] Lyrics
        [ ] [ ] [ ] [X] [ ] MensuralStaff
        [ ] [ ] [ ] [ ] [X] MensuralVoice
        [ ] [ ] [ ] [ ] [X] NoteNames
        [ ] [ ] [ ] [ ] [X] NullVoice
        [ ] [ ] [ ] [X] [ ] PetrucciStaff
        [ ] [ ] [ ] [ ] [X] PetrucciVoice
        [ ] [ ] [X] [ ] [ ] PianoStaff
        [ ] [ ] [ ] [X] [ ] RhythmicStaff
        [ ] [X] [ ] [ ] [ ] Score
        [ ] [ ] [ ] [X] [ ] Staff
        [ ] [ ] [X] [ ] [ ] StaffGroup
        [ ] [ ] [ ] [X] [ ] TabStaff
        [ ] [ ] [ ] [ ] [X] TabVoice
        [ ] [ ] [ ] [X] [ ] VaticanaStaff
        [ ] [ ] [ ] [ ] [X] VaticanaVoice
        [ ] [ ] [ ] [ ] [X] Voice

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_name',
        )

    _identity_map: typing.Dict[str, 'LilyPondContext'] = {}

    _publish_storage_format = True

    ### CONSTRUCTOR ###

    def __new__(class_, name='Voice'):
        if isinstance(name, class_):
            name = name.name
        if name in class_._identity_map:
            obj = class_._identity_map[name]
        else:
            obj = object.__new__(class_)
            class_._identity_map[name] = obj
        return obj

    ### INITIALIZER ###

    def __init__(self, name='Voice') -> None:
        from abjad.ly import contexts
        assert name in contexts
        self._name = name

    ### PUBLIC PROPERTIES ###

    @property
    def accepted_by(self) -> typing.Tuple['LilyPondContext', ...]:
        r"""
        Gets contexts accepting LilyPond context.

        ..  container:: example

            >>> context = abjad.LilyPondContext('MensuralStaff')
            >>> for accepting_context in context.accepted_by:
            ...     accepting_context
            ...
            LilyPondContext(name='Score')

            >>> for lilypond_context in abjad.LilyPondContext.list_all_contexts():
            ...     print(f'{lilypond_context.name}:')
            ...     accepted_by = lilypond_context.accepted_by
            ...     if accepted_by:
            ...         accepted_by = ',\n    '.join(_.name for _ in accepted_by)
            ...         print(f'    {accepted_by}')
            ...
            ChoirStaff:
                ChoirStaff,
                Score,
                StaffGroup
            ChordNames:
                ChoirStaff,
                GrandStaff,
                PianoStaff,
                Score,
                StaffGroup
            CueVoice:
                DrumStaff,
                GregorianTranscriptionStaff,
                KievanStaff,
                MensuralStaff,
                PetrucciStaff,
                RhythmicStaff,
                Staff,
                TabStaff,
                VaticanaStaff
            Devnull:
                Score
            DrumStaff:
                ChoirStaff,
                GrandStaff,
                PianoStaff,
                Score,
                StaffGroup
            DrumVoice:
                DrumStaff
            Dynamics:
                GrandStaff,
                PianoStaff
            FiguredBass:
                ChoirStaff,
                GrandStaff,
                PianoStaff,
                Score,
                StaffGroup
            FretBoards:
                Score,
                StaffGroup
            Global:
            GrandStaff:
                ChoirStaff,
                Score,
                StaffGroup
            GregorianTranscriptionStaff:
                Score
            GregorianTranscriptionVoice:
                GregorianTranscriptionStaff
            KievanStaff:
                Score
            KievanVoice:
                KievanStaff
            Lyrics:
                ChoirStaff,
                GrandStaff,
                PianoStaff,
                Score,
                StaffGroup
            MensuralStaff:
                Score
            MensuralVoice:
                MensuralStaff
            NoteNames:
                Score
            NullVoice:
                DrumStaff,
                GregorianTranscriptionStaff,
                KievanStaff,
                MensuralStaff,
                PetrucciStaff,
                RhythmicStaff,
                Staff,
                TabStaff,
                VaticanaStaff
            PetrucciStaff:
                Score
            PetrucciVoice:
                PetrucciStaff
            PianoStaff:
                ChoirStaff,
                Score,
                StaffGroup
            RhythmicStaff:
                ChoirStaff,
                GrandStaff,
                PianoStaff,
                Score,
                StaffGroup
            Score:
                Global
            Staff:
                ChoirStaff,
                GrandStaff,
                PianoStaff,
                Score,
                StaffGroup
            StaffGroup:
                ChoirStaff,
                Score,
                StaffGroup
            TabStaff:
                GrandStaff,
                PianoStaff,
                Score,
                StaffGroup
            TabVoice:
                TabStaff
            VaticanaStaff:
                Score
            VaticanaVoice:
                VaticanaStaff
            Voice:
                RhythmicStaff,
                Staff

        """
        from abjad.ly import contexts
        accepting_contexts = set()
        for lilypond_type, context_info in contexts.items():
            assert isinstance(context_info, dict), repr(context_info)
            if self.name in context_info['accepts']:
                accepting_context = LilyPondContext(lilypond_type)
                accepting_contexts.add(accepting_context)
        return tuple(sorted(accepting_contexts, key=lambda x: x.name))

    @property
    def accepts(self) -> typing.Tuple['LilyPondContext', ...]:
        r"""
        Gets contexts accepted by LilyPond context.

        ..  container:: example

            >>> context = abjad.LilyPondContext('MensuralStaff')
            >>> for accepted_context in context.accepts:
            ...     accepted_context
            ...
            LilyPondContext(name='CueVoice')
            LilyPondContext(name='MensuralVoice')
            LilyPondContext(name='NullVoice')

        """
        from abjad.ly import contexts
        dictionary = contexts[self.name]
        assert isinstance(dictionary, dict), repr(dictionary)
        accepts = (
            LilyPondContext(name=name) for name in dictionary['accepts']
            )
        return tuple(sorted(accepts, key=lambda x: x.name))

    @property
    def alias(self) -> typing.Optional['LilyPondContext']:
        r"""
        Gets alias of LilyPond context.

        ..  container:: example

            >>> context = abjad.LilyPondContext('MensuralStaff')
            >>> context.alias
            LilyPondContext(name='Staff')

        """
        from abjad.ly import contexts
        dictionary = contexts[self.name]
        assert isinstance(dictionary, dict)
        aliases = dictionary['aliases']
        if aliases:
            alias = tuple(aliases)[0]
            if alias not in contexts:
                return None
            return LilyPondContext(name=alias)
        return None

    @property
    def default_child(self) -> typing.Optional['LilyPondContext']:
        r"""
        Gets default child of LilyPond context.

        ..  container:: example

            >>> context = abjad.LilyPondContext('MensuralStaff')
            >>> context.default_child
            LilyPondContext(name='MensuralVoice')

            >>> for lilypond_context in abjad.LilyPondContext.list_all_contexts():
            ...     print(f'{lilypond_context.name}:')
            ...     default_child = lilypond_context.default_child
            ...     if default_child:
            ...         print(f'    {default_child.name}')
            ...
            ChoirStaff:
                Staff
            ChordNames:
            CueVoice:
            Devnull:
            DrumStaff:
                DrumVoice
            DrumVoice:
            Dynamics:
            FiguredBass:
            FretBoards:
            Global:
                Score
            GrandStaff:
                Staff
            GregorianTranscriptionStaff:
                GregorianTranscriptionVoice
            GregorianTranscriptionVoice:
            KievanStaff:
                KievanVoice
            KievanVoice:
            Lyrics:
            MensuralStaff:
                MensuralVoice
            MensuralVoice:
            NoteNames:
            NullVoice:
            PetrucciStaff:
                PetrucciVoice
            PetrucciVoice:
            PianoStaff:
                Staff
            RhythmicStaff:
                Voice
            Score:
                Staff
            Staff:
                Voice
            StaffGroup:
                Staff
            TabStaff:
                TabVoice
            TabVoice:
            VaticanaStaff:
                VaticanaVoice
            VaticanaVoice:
            Voice:

        """
        from abjad.ly import contexts
        if self.is_bottom_context:
            return None
        dictionary = contexts[self.name]
        assert isinstance(dictionary, dict), repr(dictionary)
        default_child_name = dictionary.get('default_child', None)
        if default_child_name is None:
            alias = self.alias
            if alias is not None:
                return alias.default_child
        if default_child_name and default_child_name in contexts:
            return LilyPondContext(name=default_child_name)
        return None

    @property
    def engravers(self) -> typing.Tuple[LilyPondEngraver, ...]:
        r"""
        Gets engravers belonging to LilyPond context.

        ..  container:: example

            >>> context = abjad.LilyPondContext('MensuralStaff')
            >>> for engraver in context.engravers:
            ...     engraver
            ...
            LilyPondEngraver(name='Accidental_engraver')
            LilyPondEngraver(name='Axis_group_engraver')
            LilyPondEngraver(name='Bar_engraver')
            LilyPondEngraver(name='Clef_engraver')
            LilyPondEngraver(name='Collision_engraver')
            LilyPondEngraver(name='Cue_clef_engraver')
            LilyPondEngraver(name='Custos_engraver')
            LilyPondEngraver(name='Dot_column_engraver')
            LilyPondEngraver(name='Figured_bass_engraver')
            LilyPondEngraver(name='Figured_bass_position_engraver')
            LilyPondEngraver(name='Fingering_column_engraver')
            LilyPondEngraver(name='Font_size_engraver')
            LilyPondEngraver(name='Grob_pq_engraver')
            LilyPondEngraver(name='Instrument_name_engraver')
            LilyPondEngraver(name='Key_engraver')
            LilyPondEngraver(name='Ledger_line_engraver')
            LilyPondEngraver(name='Ottava_spanner_engraver')
            LilyPondEngraver(name='Output_property_engraver')
            LilyPondEngraver(name='Piano_pedal_align_engraver')
            LilyPondEngraver(name='Piano_pedal_engraver')
            LilyPondEngraver(name='Pure_from_neighbor_engraver')
            LilyPondEngraver(name='Rest_collision_engraver')
            LilyPondEngraver(name='Script_row_engraver')
            LilyPondEngraver(name='Separating_line_group_engraver')
            LilyPondEngraver(name='Staff_collecting_engraver')
            LilyPondEngraver(name='Staff_symbol_engraver')
            LilyPondEngraver(name='Time_signature_engraver')

        """
        from abjad.ly import contexts
        engravers = set()
        dictionary = contexts[self.name]
        assert isinstance(dictionary, dict), repr(dictionary)
        for engraver_name in dictionary['consists']:
            engraver = LilyPondEngraver(name=engraver_name)
            engravers.add(engraver)
        engravers_ = tuple(sorted(engravers, key=lambda x: x.name))
        return engravers_

    @property
    def grobs(self) -> typing.Tuple[LilyPondGrob, ...]:
        r"""
        Gets grobs created by LilyPond context.

        ..  container:: example

            >>> context = abjad.LilyPondContext('MensuralStaff')
            >>> for grob in context.grobs:
            ...     grob
            ...
            LilyPondGrob(name='Accidental')
            LilyPondGrob(name='AccidentalCautionary')
            LilyPondGrob(name='AccidentalPlacement')
            LilyPondGrob(name='AccidentalSuggestion')
            LilyPondGrob(name='BarLine')
            LilyPondGrob(name='BassFigure')
            LilyPondGrob(name='BassFigureAlignment')
            LilyPondGrob(name='BassFigureAlignmentPositioning')
            LilyPondGrob(name='BassFigureBracket')
            LilyPondGrob(name='BassFigureContinuation')
            LilyPondGrob(name='BassFigureLine')
            LilyPondGrob(name='Clef')
            LilyPondGrob(name='ClefModifier')
            LilyPondGrob(name='CueClef')
            LilyPondGrob(name='CueEndClef')
            LilyPondGrob(name='Custos')
            LilyPondGrob(name='DotColumn')
            LilyPondGrob(name='FingeringColumn')
            LilyPondGrob(name='InstrumentName')
            LilyPondGrob(name='KeyCancellation')
            LilyPondGrob(name='KeySignature')
            LilyPondGrob(name='LedgerLineSpanner')
            LilyPondGrob(name='NoteCollision')
            LilyPondGrob(name='OttavaBracket')
            LilyPondGrob(name='PianoPedalBracket')
            LilyPondGrob(name='RestCollision')
            LilyPondGrob(name='ScriptRow')
            LilyPondGrob(name='SostenutoPedal')
            LilyPondGrob(name='SostenutoPedalLineSpanner')
            LilyPondGrob(name='StaffSpacing')
            LilyPondGrob(name='StaffSymbol')
            LilyPondGrob(name='SustainPedal')
            LilyPondGrob(name='SustainPedalLineSpanner')
            LilyPondGrob(name='TimeSignature')
            LilyPondGrob(name='UnaCordaPedal')
            LilyPondGrob(name='UnaCordaPedalLineSpanner')
            LilyPondGrob(name='VerticalAxisGroup')

        """
        grobs: typing.Set[LilyPondGrob] = set()
        for engraver in self.engravers:
            grobs.update(engraver.grobs)
        return tuple(sorted(grobs, key=lambda x: x.name))

    @property
    def is_bottom_context(self) -> bool:
        r"""
        Is true if LilyPond context is a bottom context.

        ..  container:: example

            >>> for lilypond_context in abjad.LilyPondContext.list_all_contexts():
            ...     is_bottom_context = 'X' if lilypond_context.is_bottom_context else ' '
            ...     print(f'[{is_bottom_context}] {lilypond_context.name}')
            ...
            [ ] ChoirStaff
            [X] ChordNames
            [X] CueVoice
            [X] Devnull
            [ ] DrumStaff
            [X] DrumVoice
            [X] Dynamics
            [X] FiguredBass
            [X] FretBoards
            [ ] Global
            [ ] GrandStaff
            [ ] GregorianTranscriptionStaff
            [X] GregorianTranscriptionVoice
            [ ] KievanStaff
            [X] KievanVoice
            [X] Lyrics
            [ ] MensuralStaff
            [X] MensuralVoice
            [X] NoteNames
            [X] NullVoice
            [ ] PetrucciStaff
            [X] PetrucciVoice
            [ ] PianoStaff
            [ ] RhythmicStaff
            [ ] Score
            [ ] Staff
            [ ] StaffGroup
            [ ] TabStaff
            [X] TabVoice
            [ ] VaticanaStaff
            [X] VaticanaVoice
            [X] Voice

        """
        if not self.accepts:
            return True
        return False

    @property
    def is_custom(self) -> bool:
        r"""
        Is true if LilyPond context is user-created.

        ..  container:: example

            >>> context = abjad.LilyPondContext('MensuralStaff')
            >>> context.is_custom
            False

        """
        from abjad.ly import contexts
        dictionary = contexts[self.name]
        assert isinstance(dictionary, dict), repr(dictionary)
        return bool(dictionary.get('is_custom', False))

    @property
    def is_global_context(self) -> bool:
        r"""
        Is true if LilyPond context is a global context.

        ..  container:: example

            >>> for lilypond_context in abjad.LilyPondContext.list_all_contexts():
            ...     is_global_context = 'X' if lilypond_context.is_global_context else ' '
            ...     print(f'[{is_global_context}] {lilypond_context.name}')
            ...
            [ ] ChoirStaff
            [ ] ChordNames
            [ ] CueVoice
            [ ] Devnull
            [ ] DrumStaff
            [ ] DrumVoice
            [ ] Dynamics
            [ ] FiguredBass
            [ ] FretBoards
            [X] Global
            [ ] GrandStaff
            [ ] GregorianTranscriptionStaff
            [ ] GregorianTranscriptionVoice
            [ ] KievanStaff
            [ ] KievanVoice
            [ ] Lyrics
            [ ] MensuralStaff
            [ ] MensuralVoice
            [ ] NoteNames
            [ ] NullVoice
            [ ] PetrucciStaff
            [ ] PetrucciVoice
            [ ] PianoStaff
            [ ] RhythmicStaff
            [ ] Score
            [ ] Staff
            [ ] StaffGroup
            [ ] TabStaff
            [ ] TabVoice
            [ ] VaticanaStaff
            [ ] VaticanaVoice
            [ ] Voice

        """
        if not self.accepts:
            return False
        elif self is type(self)('Global'):
            return True
        elif self.alias is type(self)('Global'):
            return True
        return False

    @property
    def is_score_context(self) -> bool:
        r"""
        Is true if LilyPond context is a score context.

        ..  container:: example

            >>> for lilypond_context in abjad.LilyPondContext.list_all_contexts():
            ...     is_score_context = 'X' if lilypond_context.is_score_context else ' '
            ...     print(f'[{is_score_context}] {lilypond_context.name}')
            ...
            [ ] ChoirStaff
            [ ] ChordNames
            [ ] CueVoice
            [ ] Devnull
            [ ] DrumStaff
            [ ] DrumVoice
            [ ] Dynamics
            [ ] FiguredBass
            [ ] FretBoards
            [ ] Global
            [ ] GrandStaff
            [ ] GregorianTranscriptionStaff
            [ ] GregorianTranscriptionVoice
            [ ] KievanStaff
            [ ] KievanVoice
            [ ] Lyrics
            [ ] MensuralStaff
            [ ] MensuralVoice
            [ ] NoteNames
            [ ] NullVoice
            [ ] PetrucciStaff
            [ ] PetrucciVoice
            [ ] PianoStaff
            [ ] RhythmicStaff
            [X] Score
            [ ] Staff
            [ ] StaffGroup
            [ ] TabStaff
            [ ] TabVoice
            [ ] VaticanaStaff
            [ ] VaticanaVoice
            [ ] Voice

        """
        if not self.accepts:
            return False
        elif self is type(self)('Score'):
            return True
        elif self.alias is type(self)('Score'):
            return True
        return False

    @property
    def is_staff_context(self) -> bool:
        r"""
        Is true if LilyPond context is a staff context.

        ..  container:: example

            >>> for lilypond_context in abjad.lilypondnames.LilyPondContext.list_all_contexts():
            ...     is_staff_context = 'X' if lilypond_context.is_staff_context else ' '
            ...     print(f'[{is_staff_context}] {lilypond_context.name}')
            ...
            [ ] ChoirStaff
            [ ] ChordNames
            [ ] CueVoice
            [ ] Devnull
            [X] DrumStaff
            [ ] DrumVoice
            [ ] Dynamics
            [ ] FiguredBass
            [ ] FretBoards
            [ ] Global
            [ ] GrandStaff
            [X] GregorianTranscriptionStaff
            [ ] GregorianTranscriptionVoice
            [X] KievanStaff
            [ ] KievanVoice
            [ ] Lyrics
            [X] MensuralStaff
            [ ] MensuralVoice
            [ ] NoteNames
            [ ] NullVoice
            [X] PetrucciStaff
            [ ] PetrucciVoice
            [ ] PianoStaff
            [X] RhythmicStaff
            [ ] Score
            [X] Staff
            [ ] StaffGroup
            [X] TabStaff
            [ ] TabVoice
            [X] VaticanaStaff
            [ ] VaticanaVoice
            [ ] Voice

        """
        if not self.accepts:
            return False
        elif self is type(self)('Staff'):
            return True
        elif self.alias is type(self)('Staff'):
            return True
        return False

    @property
    def is_staff_group_context(self) -> bool:
        r"""
        Is true if LilyPond context is a staff group context.

        ..  container:: example

            >>> for lilypond_context in abjad.lilypondnames.LilyPondContext.list_all_contexts():
            ...     is_staff_group_context = 'X' if lilypond_context.is_staff_group_context else ' '
            ...     print(f'[{is_staff_group_context}] {lilypond_context.name}')
            ...
            [X] ChoirStaff
            [ ] ChordNames
            [ ] CueVoice
            [ ] Devnull
            [ ] DrumStaff
            [ ] DrumVoice
            [ ] Dynamics
            [ ] FiguredBass
            [ ] FretBoards
            [ ] Global
            [X] GrandStaff
            [ ] GregorianTranscriptionStaff
            [ ] GregorianTranscriptionVoice
            [ ] KievanStaff
            [ ] KievanVoice
            [ ] Lyrics
            [ ] MensuralStaff
            [ ] MensuralVoice
            [ ] NoteNames
            [ ] NullVoice
            [ ] PetrucciStaff
            [ ] PetrucciVoice
            [X] PianoStaff
            [ ] RhythmicStaff
            [ ] Score
            [ ] Staff
            [X] StaffGroup
            [ ] TabStaff
            [ ] TabVoice
            [ ] VaticanaStaff
            [ ] VaticanaVoice
            [ ] Voice

        """
        return not any([
            self.is_global_context,
            self.is_score_context,
            self.is_staff_context,
            self.is_bottom_context,
            ])

    @property
    def name(self) -> str:
        r"""
        Gets name of LilyPond context.

        ..  container:: example

            >>> context = abjad.LilyPondContext('MensuralStaff')
            >>> context.name
            'MensuralStaff'

        """
        return self._name

    @property
    def property_names(self) -> typing.Tuple[str, ...]:
        r"""
        Gets property names of LilyPond context.

        ..  container:: example

            >>> context = abjad.LilyPondContext('MensuralStaff')
            >>> for property_name in context.property_names:
            ...     property_name
            ...
            'accidentalGrouping'
            'autoAccidentals'
            'autoCautionaries'
            'busyGrobs'
            'clefGlyph'
            'clefPosition'
            'clefTransposition'
            'clefTranspositionStyle'
            'createKeyOnClefChange'
            'createSpacing'
            'cueClefGlyph'
            'cueClefPosition'
            'cueClefTransposition'
            'cueClefTranspositionStyle'
            'currentCommandColumn'
            'currentMusicalColumn'
            'explicitClefVisibility'
            'explicitCueClefVisibility'
            'explicitKeySignatureVisibility'
            'extraNatural'
            'figuredBassAlterationDirection'
            'figuredBassCenterContinuations'
            'figuredBassFormatter'
            'fontSize'
            'forbidBreak'
            'forceClef'
            'harmonicAccidentals'
            'hasAxisGroup'
            'hasStaffSpacing'
            'ignoreFiguredBassRest'
            'implicitBassFigures'
            'initialTimeSignatureVisibility'
            'instrumentName'
            'internalBarNumber'
            'keepAliveInterfaces'
            'keyAlterationOrder'
            'keyAlterations'
            'lastKeyAlterations'
            'localAlterations'
            'middleCClefPosition'
            'middleCCuePosition'
            'middleCOffset'
            'ottavation'
            'partialBusy'
            'pedalSostenutoStrings'
            'pedalSostenutoStyle'
            'pedalSustainStrings'
            'pedalSustainStyle'
            'pedalUnaCordaStrings'
            'pedalUnaCordaStyle'
            'printKeyCancellation'
            'shortInstrumentName'
            'shortVocalName'
            'stavesFound'
            'timeSignatureFraction'
            'tonic'
            'useBassFigureExtenders'
            'vocalName'
            'whichBar'

        """
        property_names: typing.Set[str] = set()
        for engraver in self.engravers:
            property_names.update(engraver.property_names)
        return tuple(sorted(property_names))

    ### PUBLIC METHODS ###

    @staticmethod
    def list_all_contexts() -> typing.Tuple['LilyPondContext', ...]:
        r"""
        Lists all contexts.

        ..  container:: example

            >>> for lilypond_context in abjad.LilyPondContext.list_all_contexts():
            ...     lilypond_context
            ...
            LilyPondContext(name='ChoirStaff')
            LilyPondContext(name='ChordNames')
            LilyPondContext(name='CueVoice')
            LilyPondContext(name='Devnull')
            LilyPondContext(name='DrumStaff')
            LilyPondContext(name='DrumVoice')
            LilyPondContext(name='Dynamics')
            LilyPondContext(name='FiguredBass')
            LilyPondContext(name='FretBoards')
            LilyPondContext(name='Global')
            LilyPondContext(name='GrandStaff')
            LilyPondContext(name='GregorianTranscriptionStaff')
            LilyPondContext(name='GregorianTranscriptionVoice')
            LilyPondContext(name='KievanStaff')
            LilyPondContext(name='KievanVoice')
            LilyPondContext(name='Lyrics')
            LilyPondContext(name='MensuralStaff')
            LilyPondContext(name='MensuralVoice')
            LilyPondContext(name='NoteNames')
            LilyPondContext(name='NullVoice')
            LilyPondContext(name='PetrucciStaff')
            LilyPondContext(name='PetrucciVoice')
            LilyPondContext(name='PianoStaff')
            LilyPondContext(name='RhythmicStaff')
            LilyPondContext(name='Score')
            LilyPondContext(name='Staff')
            LilyPondContext(name='StaffGroup')
            LilyPondContext(name='TabStaff')
            LilyPondContext(name='TabVoice')
            LilyPondContext(name='VaticanaStaff')
            LilyPondContext(name='VaticanaVoice')
            LilyPondContext(name='Voice')

        """
        from abjad.ly import contexts
        return tuple(LilyPondContext(name=name) for name in sorted(contexts))

    @classmethod
    def register(
        class_,
        accepted_by: typing.List[str] = None,
        accepts=None,
        alias: typing.Union[str, 'LilyPondContext'] = None,
        consists=None,
        default_child=None,
        denies=None,
        name: str = None,
        removes: typing.List[str] = None,
        ) -> 'LilyPondContext':
        r"""
        Registers a new context.

        ..  container:: example

            >>> custom_context = abjad.LilyPondContext.register(
            ...     accepted_by=['Score', 'StaffGroup'],
            ...     alias='Staff',
            ...     name='BowingStaff',
            ...     removes=['Note_heads_engraver'],
            ...     )
            >>> print(format(custom_context))
            abjad.LilyPondContext(
                name='BowingStaff',
                )

            >>> custom_context.is_custom
            True

            >>> for engraver in custom_context.engravers:
            ...     engraver
            ...
            LilyPondEngraver(name='Accidental_engraver')
            LilyPondEngraver(name='Axis_group_engraver')
            LilyPondEngraver(name='Bar_engraver')
            LilyPondEngraver(name='Clef_engraver')
            LilyPondEngraver(name='Collision_engraver')
            LilyPondEngraver(name='Cue_clef_engraver')
            LilyPondEngraver(name='Dot_column_engraver')
            LilyPondEngraver(name='Figured_bass_engraver')
            LilyPondEngraver(name='Figured_bass_position_engraver')
            LilyPondEngraver(name='Fingering_column_engraver')
            LilyPondEngraver(name='Font_size_engraver')
            LilyPondEngraver(name='Grob_pq_engraver')
            LilyPondEngraver(name='Instrument_name_engraver')
            LilyPondEngraver(name='Key_engraver')
            LilyPondEngraver(name='Ledger_line_engraver')
            LilyPondEngraver(name='Ottava_spanner_engraver')
            LilyPondEngraver(name='Output_property_engraver')
            LilyPondEngraver(name='Piano_pedal_align_engraver')
            LilyPondEngraver(name='Piano_pedal_engraver')
            LilyPondEngraver(name='Pure_from_neighbor_engraver')
            LilyPondEngraver(name='Rest_collision_engraver')
            LilyPondEngraver(name='Script_row_engraver')
            LilyPondEngraver(name='Separating_line_group_engraver')
            LilyPondEngraver(name='Staff_collecting_engraver')
            LilyPondEngraver(name='Staff_symbol_engraver')
            LilyPondEngraver(name='Time_signature_engraver')

            >>> score_context = abjad.LilyPondContext('Score')
            >>> custom_context in score_context.accepts
            True

            >>> custom_context.unregister()

        """
        from abjad.ly import contexts
        assert name not in contexts
        context_entry: typing.Dict = {}
        context_entry['accepts'] = set()
        context_entry['consists'] = set()
        context_entry['is_custom'] = True
        if alias is not None:
            if not isinstance(alias, class_):
                alias_ = class_(name=alias)
            else:
                alias_ = alias
            assert isinstance(alias_, class_)
            context_entry['accepts'].update(_.name for _ in alias_.accepts)
            context_entry['consists'].update(_.name for _ in alias_.engravers)
            context_entry['aliases'] = set([alias_.name])
        if accepts:
            for x in accepts:
                if not isinstance(x, class_):
                    x = class_(name=x)
                assert isinstance(x, class_)
                context_entry['accepts'].add(x.name)
        if denies:
            for x in denies:
                if not isinstance(x, class_):
                    x = class_(name=x)
                assert isinstance(x, class_)
                if x.name in context_entry['accepts']:
                    context_entry['accepts'].remove(x.name)
        if consists:
            for x in consists:
                if not isinstance(x, LilyPondEngraver):
                    x = LilyPondEngraver(name=x)
                assert isinstance(x, LilyPondEngraver)
                context_entry['consists'].add(x.name)
        if removes:
            for x in removes:
                if not isinstance(x, LilyPondEngraver):
                    x = LilyPondEngraver(name=x)
                assert isinstance(x, LilyPondEngraver)
                if x.name in context_entry['consists']:
                    context_entry['consists'].remove(x.name)
        if default_child is not None:
            if not isinstance(default_child, class_):
                default_child = class_(name=default_child)
            assert isinstance(default_child, class_)
            context_entry['default_child'] = default_child.name
        accepting_contexts = set()
        if accepted_by:
            for x in accepted_by:
                if not isinstance(x, class_):
                    x = class_(name=x)
                assert isinstance(x, class_)
                accepting_contexts.add(x.name)
        assert isinstance(name, str)
        contexts[name] = context_entry
        for accepting_context in accepting_contexts:
            dictionary = contexts[accepting_context]
            assert isinstance(dictionary, dict)
            dictionary['accepts'].add(name)
        custom_context = class_(name=name)
        return custom_context

    def unregister(
        self,
        context=None,
        ) -> None:
        r"""
        Unregisters custom context.

        ..  container:: example

            >>> custom_context = abjad.LilyPondContext.register(
            ...     accepted_by=['Score', 'StaffGroup'],
            ...     alias='Staff',
            ...     name='FingeringStaff',
            ...     )

            >>> score_context = abjad.LilyPondContext('Score')
            >>> for accepted_context in score_context.accepts:
            ...     accepted_context
            ...
            LilyPondContext(name='ChoirStaff')
            LilyPondContext(name='ChordNames')
            LilyPondContext(name='Devnull')
            LilyPondContext(name='DrumStaff')
            LilyPondContext(name='FiguredBass')
            LilyPondContext(name='FingeringStaff')
            LilyPondContext(name='FretBoards')
            LilyPondContext(name='GrandStaff')
            LilyPondContext(name='GregorianTranscriptionStaff')
            LilyPondContext(name='KievanStaff')
            LilyPondContext(name='Lyrics')
            LilyPondContext(name='MensuralStaff')
            LilyPondContext(name='NoteNames')
            LilyPondContext(name='PetrucciStaff')
            LilyPondContext(name='PianoStaff')
            LilyPondContext(name='RhythmicStaff')
            LilyPondContext(name='Staff')
            LilyPondContext(name='StaffGroup')
            LilyPondContext(name='TabStaff')
            LilyPondContext(name='VaticanaStaff')

            >>> custom_context.unregister()

            >>> score_context = abjad.LilyPondContext('Score')
            >>> for accepted_context in score_context.accepts:
            ...     accepted_context
            ...
            LilyPondContext(name='ChoirStaff')
            LilyPondContext(name='ChordNames')
            LilyPondContext(name='Devnull')
            LilyPondContext(name='DrumStaff')
            LilyPondContext(name='FiguredBass')
            LilyPondContext(name='FretBoards')
            LilyPondContext(name='GrandStaff')
            LilyPondContext(name='GregorianTranscriptionStaff')
            LilyPondContext(name='KievanStaff')
            LilyPondContext(name='Lyrics')
            LilyPondContext(name='MensuralStaff')
            LilyPondContext(name='NoteNames')
            LilyPondContext(name='PetrucciStaff')
            LilyPondContext(name='PianoStaff')
            LilyPondContext(name='RhythmicStaff')
            LilyPondContext(name='Staff')
            LilyPondContext(name='StaffGroup')
            LilyPondContext(name='TabStaff')
            LilyPondContext(name='VaticanaStaff')

        """
        from abjad.ly import contexts
        assert self.is_custom
        del(contexts[self.name])
        del(self._identity_map[self.name])
        for lilypond_type, context_info in contexts.items():
            assert isinstance(context_info, dict), repr(context_info)
            set_ = context_info['accepts']
            assert isinstance(set_, set), repr(set_)
            if self.name in set_:
                set_.remove(self.name)



def timespan_2_contains_timespan_1_improperly(
    timespan_1=None,
    timespan_2=None,
    hold=False,
    ):
    r"""
    Makes time relation indicating that ``timespan_2`` contains
    ``timespan_1`` improperly.

    ..  container:: example

        >>> relation = abjad.timespans.timespan_2_contains_timespan_1_improperly()
        >>> abjad.f(relation)
        abjad.timespans.TimespanTimespanTimeRelation(
            inequality=abjad.timespans.CompoundInequality(
                [
                    abjad.TimespanInequality('timespan_2.start_offset <= timespan_1.start_offset'),
                    abjad.TimespanInequality('timespan_1.stop_offset <= timespan_2.stop_offset'),
                    ],
                logical_operator='and',
                ),
            )

    ..  container:: example

        >>> staff = abjad.Staff(
        ...     r"c'8. \p \< fs'16 a'4 af'8 \f \> g'8 ~ g'16 f' e' ef' \p",
        ...     )
        >>> timespan_1 = abjad.Timespan((1, 4), (3, 8))
        >>> abjad.show(staff) # doctest: +SKIP

        >>> offset_lists = staff[:]._get_offset_lists()
        >>> time_relation = abjad.timespans.timespan_2_contains_timespan_1_improperly(timespan_1=timespan_1)
        >>> start_index, stop_index = time_relation.get_offset_indices(*offset_lists)
        >>> selected_notes = staff[start_index:stop_index]
        >>> selected_notes
        Selection([Note("a'4")])

        >>> abjad.label(selected_notes).color_leaves('red')
        >>> abjad.show(staff) # doctest: +SKIP

    Returns time relation or boolean.
    """
    from abjad import timespans

    inequality = timespans.CompoundInequality([
        'timespan_2.start_offset <= timespan_1.start_offset',
        'timespan_1.stop_offset <= timespan_2.stop_offset',
        ])

    time_relation = timespans.TimespanTimespanTimeRelation(
        inequality,
        timespan_1=timespan_1,
        timespan_2=timespan_2,
        )

    if time_relation.is_fully_loaded and not hold:
        return time_relation()
    else:
        return time_relation



def inspect(client):
    r"""
    Makes inspection agent.

    ..  container:: example

        Example staff:

        >>> staff = abjad.Staff("c'4 e'4 d'4 f'4")
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'4
                e'4
                d'4
                f'4
            }

    ..  container:: example

        Gets duration of first note in staff:

        >>> abjad.inspect(staff[0]).duration()
        Duration(1, 4)

    ..  container:: example

        Returns inspection agent:

        >>> abjad.inspect(staff)
        Inspection(client=Staff("c'4 e'4 d'4 f'4"))

    """
    import abjad
    return abjad.Inspection(client=client)



import collections
from abjad.system.AbjadObject import AbjadObject


class VerticalMoment(AbjadObject):
    r'''
    Vertical moment.

    ..  container:: example

        >>> score = abjad.Score()
        >>> staff_group = abjad.StaffGroup()
        >>> staff_group.lilypond_type = 'PianoStaff'
        >>> staff_group.append(abjad.Staff("c'4 e'4 d'4 f'4"))
        >>> staff_group.append(abjad.Staff(r"""\clef "bass" g2 f2"""))
        >>> score.append(staff_group)
        >>> abjad.show(score) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(score)
            \new Score
            <<
                \new PianoStaff
                <<
                    \new Staff
                    {
                        c'4
                        e'4
                        d'4
                        f'4
                    }
                    \new Staff
                    {
                        \clef "bass"
                        g2
                        f2
                    }
                >>
            >>

        >>> for moment in abjad.iterate(score).vertical_moments():
        ...     moment
        ...
        VerticalMoment(0, <<2>>)
        VerticalMoment(1/4, <<2>>)
        VerticalMoment(1/2, <<2>>)
        VerticalMoment(3/4, <<2>>)

    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Selections'

    __slots__ = (
        '_components',
        '_governors',
        '_offset',
        )

    ### INITIALIZER ###

    def __init__(self, components=None, offset=None):
        import abjad
        if components is None:
            self._offset = offset
            self._components = ()
            self._governors = ()
        else:
            governors, components = self._from_offset(components, offset)
            offset = abjad.Offset(offset)
            self._offset = offset
            assert isinstance(governors, collections.Iterable)
            governors = tuple(governors)
            self._governors = governors
            assert isinstance(components, collections.Iterable)
            components = list(components)
            components.sort(
                key=lambda _: abjad.inspect(_).parentage().score_index)
        self._components = components

    ### SPECIAL METHODS ###

    def __eq__(self, argument):
        """
        Is true when ``argument`` is a vertical moment with the same
        components as this vertical moment.

        Returns true or false.
        """
        if isinstance(argument, VerticalMoment):
            if len(self) == len(argument):
                for c, d in zip(self.components, argument.components):
                    if c is not d:
                        return False
                else:
                    return True
        return False

    def __hash__(self):
        """
        Hases vertical moment.

        Redefined in tandem with __eq__.
        """
        if self.components:
            return hash(tuple([id(_) for _ in self.components]))
        return 0

    def __len__(self):
        """
        Length of vertical moment.

        Defined equal to the number of components in vertical moment.

        Returns nonnegative integer.
        """
        return len(self.components)

    def __repr__(self):
        """
        Gets interpreter representation of vertical moment.

        Returns string.
        """
        if not self.components:
            return f'{type(self).__name__}()'
        result = '{}({}, <<{}>>)'
        result = result.format(
            type(self).__name__,
            str(self.offset),
            len(self.leaves),
            )
        return result

    ### PRIVATE METHODS ###

    @staticmethod
    def _find_index(container, offset):
        """
        Based off of Python's bisect.bisect() function.
        """
        import abjad
        lo = 0
        hi = len(container)
        while lo < hi:
            mid = (lo + hi) // 2
            start_offset = abjad.inspect(container[mid]).timespan().start_offset
            stop_offset = abjad.inspect(container[mid]).timespan().stop_offset
            if start_offset <= offset < stop_offset:
                lo = mid + 1
            # if container[mid] is of nonzero duration
            elif start_offset < stop_offset:
                hi = mid
            # container[mid] is of zero duration so we skip it
            else:
                lo = mid + 1
        return lo - 1

    @staticmethod
    def _from_offset(argument, offset):
        import abjad
        offset = abjad.Offset(offset)
        governors = []
        prototype = (list, tuple, abjad.Selection)
        message = f'must be component or of Abjad components: {argument!r}.'
        if isinstance(argument, abjad.Component):
            governors.append(argument)
        elif isinstance(argument, prototype):
            for x in argument:
                if isinstance(x, abjad.Component):
                    governors.append(x)
                else:
                    raise TypeError(message)
        else:
            raise TypeError(message)
        governors.sort(
            key=lambda x: abjad.inspect(x).parentage().score_index)
        governors = tuple(governors)
        components = []
        for governor in governors:
            components.extend(VerticalMoment._recurse(governor, offset))
        components.sort(
            key=lambda x: abjad.inspect(x).parentage().score_index)
        components = tuple(components)
        return governors, components

    def _get_format_specification(self):
        import abjad
        return abjad.FormatSpecification(client=self)

    @staticmethod
    def _recurse(component, offset):
        import abjad
        result = []
        if (abjad.inspect(component).timespan().start_offset <=
            offset < abjad.inspect(component).timespan().stop_offset):
            result.append(component)
            if hasattr(component, 'components'):
                if component.is_simultaneous:
                    for x in component:
                        result.extend(VerticalMoment._recurse(x, offset))
                else:
                    child = component[
                        VerticalMoment._find_index(component, offset)]
                    result.extend(VerticalMoment._recurse(child, offset))
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def attack_count(self):
        """
        Positive integer number of pitch carriers starting at vertical
        moment.
        """
        import abjad
        attack_carriers = []
        for leaf in self.start_leaves:
            if isinstance(leaf, (abjad.Note, abjad.Chord)):
                attack_carriers.append(leaf)
        return len(attack_carriers)

    @property
    def components(self):
        """
        Tuple of zero or more components happening at vertical moment.

        It is always the case that ``self.components =
        self.overlap_components + self.start_components``.
        """
        return self._components

    @property
    def governors(self):
        """
        Tuple of one or more containers in which vertical moment is evaluated.
        """
        return self._governors

    @property
    def leaves(self):
        """
        Tuple of zero or more leaves at vertical moment.
        """
        import abjad
        result = []
        for component in self.components:
            if isinstance(component, abjad.Leaf):
                result.append(component)
        result = abjad.select(result)
        return result

    @property
    def measures(self):
        """
        Tuple of zero or more measures at vertical moment.
        """
        import abjad
        result = []
        for component in self.components:
            if isinstance(component, abjad.Measure):
                result.append(component)
        result = tuple(result)
        return result

    @property
    def next_vertical_moment(self):
        """
        Reference to next vertical moment forward in time.
        """
        import abjad
        candidate_shortest_leaf = self.leaves[0]
        for leaf in self.leaves[1:]:
            if (abjad.inspect(leaf).timespan().stop_offset <
                abjad.inspect(candidate_shortest_leaf).timespan().stop_offset):
                candidate_shortest_leaf = leaf
        next_leaf = candidate_shortest_leaf._get_in_my_logical_voice(
            1, prototype=abjad.Leaf)
        next_vertical_moment = next_leaf._get_vertical_moment()
        return next_vertical_moment

    @property
    def notes(self):
        """
        Tuple of zero or more notes at vertical moment.
        """
        import abjad
        result = []
        prototype = (abjad.Note,)
        for component in self.components:
            if isinstance(component, prototype):
                result.append(component)
        result = tuple(result)
        return result

    @property
    def notes_and_chords(self):
        """
        Tuple of zero or more notes and chords at vertical moment.
        """
        import abjad
        result = []
        prototype = (abjad.Chord, abjad.Note)
        for component in self.components:
            if isinstance(component, prototype):
                result.append(component)
        result = tuple(result)
        return result

    @property
    def offset(self):
        """
        Rational-valued score offset at which vertical moment is evaluated.
        """
        return self._offset

    @property
    def overlap_components(self):
        """
        Tuple of components in vertical moment starting before vertical
        moment, ordered by score index.
        """
        result = []
        for component in self.components:
            if component.start < self.offset:
                result.append(component)
        result = tuple(result)
        return result

    @property
    def overlap_leaves(self):
        """
        Tuple of leaves in vertical moment starting before vertical moment,
        ordered by score index.
        """
        import abjad
        result = [x for x in self.overlap_components
            if isinstance(x, abjad.Leaf)]
        result = tuple(result)
        return result

    @property
    def overlap_measures(self):
        """
        Tuple of measures in vertical moment starting before vertical
        moment, ordered by score index.
        """
        import abjad
        result = self.overlap_components
        result = [_ for _ in result if isinstance(_, abjad.Measure)]
        result = tuple(result)
        return result

    @property
    def overlap_notes(self):
        """
        Tuple of notes in vertical moment starting before vertical moment,
        ordered by score index.
        """
        import abjad
        result = self.overlap_components
        result = [_ for _ in result if isinstance(_, abjad.Note)]
        result = tuple(result)
        return result

    @property
    def previous_vertical_moment(self):
        """
        Reference to previous vertical moment backward in time.
        """
        import abjad
        if self.offset == 0:
            raise IndexError
        most_recent_start_offset = abjad.Offset(0)
        token_leaf = None
        for leaf in self.leaves:
            #print ''
            #print leaf
            leaf_start = abjad.inspect(leaf).timespan().start_offset
            if leaf_start < self.offset:
                #print 'found leaf starting before this moment ...'
                if most_recent_start_offset <= leaf_start:
                    most_recent_start_offset = leaf_start
                    token_leaf = leaf
            else:
                #print 'found leaf starting on this moment ...'
                try:
                    previous_leaf = leaf._get_in_my_logical_voice(
                        -1, prototype=abjad.Leaf)
                    start = abjad.inspect(previous_leaf).timespan().start_offset
                    #print previous_leaf, start
                    if most_recent_start_offset <= start:
                        most_recent_start_offset = start
                        token_leaf = previous_leaf
                except IndexError:
                    pass
        #print 'token_leaf is %s ...' % token_leaf
        if token_leaf is None:
            token_leaf = leaf
            #print 'token_leaf is %s ...' % token_leaf
        previous_vertical_moment = token_leaf._get_vertical_moment()
        return previous_vertical_moment

    @property
    def start_components(self):
        """
        Tuple of components in vertical moment starting with at vertical
        moment, ordered by score index.
        """
        import abjad
        result = []
        for component in self.components:
            if abjad.inspect(
                component).timespan().start_offset == self.offset:
                result.append(component)
        result = tuple(result)
        return result

    @property
    def start_leaves(self):
        """
        Tuple of leaves in vertical moment starting with vertical moment,
        ordered by score index.
        """
        import abjad
        result = [x for x in self.start_components
            if isinstance(x, abjad.Leaf)]
        result = tuple(result)
        return result

    @property
    def start_notes(self):
        """
        Tuple of notes in vertical moment starting with vertical moment,
        ordered by score index.
        """
        import abjad
        result = [x for x in self.start_components
            if isinstance(x, abjad.Note)]
        result = tuple(result)
        return result



import os
import types
from abjad.system.AbjadObject import AbjadObject


class ImportManager(AbjadObject):
    """
    Imports structured packages.
    """

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Managers'

    __slots__ = ()

    ### PRIVATE METHODS ###

    @staticmethod
    def _get_public_function_names_in_module(module_file):
        """
        Collects and returns all public functions defined in module_file.
        """
        result = []
        module_file = module_file.replace(os.sep, '.')
        mod = __import__(module_file, fromlist=['*'])
        for key, value in list(vars(mod).items()):
            if not key.startswith('_'):
                # handle public function decorated with @require
                module_name = None
                try:
                    if getattr(value, 'func_closure', None):
                        module_name = getattr(
                            value.func_closure[1].cell_contents,
                            '__module__',
                            None,
                            )
                    # handle plain old function
                    else:
                        module_name = getattr(value, '__module__', None)
                except:
                    pass
                if module_name == module_file:
                    result.append(value)
        return result

    @staticmethod
    def _import_contents_of_public_packages_in_path_into_namespace(
        path,
        namespace,
        ):
        """
        Inspects the top level of path.

        Finds public class packages and imports class package
        contents into namespace.

        Does not inspect lower levels of path.
        """
        package_path = ImportManager._split_package_path(path)
        for name in os.listdir(path):
            fullname = os.path.join(path, name)
            if os.path.isdir(fullname):
                if name[0].isupper() and \
                    os.path.exists(os.path.join(fullname, '__init__.py')) and \
                    os.path.exists(os.path.join(fullname, '%s.py' % name)):
                    class_package = '.'.join([package_path, name])
                    class_module = '.'.join([class_package, name])
                    public_names = \
                        ImportManager._get_public_function_names_in_module(
                            class_module)
                    for public_name in public_names:
                        namespace[public_name.__name__] = public_name

    @staticmethod
    def _split_package_path(path):
        outer, inner = path, None
        while os.path.exists(os.path.join(outer, '__init__.py')):
            outer, inner = os.path.split(outer)
        package_path = os.path.relpath(path, outer)
        package_path = package_path.replace(os.sep, '.')
        return package_path

    ### PUBLIC METHODS ###

    @staticmethod
    def import_material_packages(
        path,
        namespace,
        ):
        """
        Imports public materials from ``path`` into ``namespace``.

        This is the custom function that all AbjadIDE-managed scores may use to
        import public materials on startup.
        """
        prefix = 'definition'
        package_path = ImportManager._split_package_path(path)
        for name in os.listdir(path):
            if not os.path.isdir(os.path.join(path, name)):
                continue
            elif name in ('.svn', '.git', 'test', '__pycache__'):
                continue
            initializer_file_path = os.path.join(
                path,
                name,
                '__init__.py',
                )
            if not os.path.exists(initializer_file_path):
                continue
            output_file_path = os.path.join(
                path,
                name,
                '{}.py'.format(prefix),
                )
            if not os.path.exists(output_file_path):
                continue
            output_module_path = '.'.join((
                package_path,
                name,
                prefix,
                ))
            output_module = __import__(output_module_path, fromlist=['*'])
            if name in dir(output_module):
                namespace[name] = getattr(output_module, name)
        if 'system' in namespace:
            del(namespace['system'])
        if ImportManager.__name__ in namespace:
            del(namespace[ImportManager.__name__])

    @staticmethod
    def import_nominative_modules(
        path,
        namespace,
        ):
        """
        Imports nominative modules from ``path`` into ``namespace``.
        """
        package_path = ImportManager._split_package_path(path)
        for name in os.listdir(path):
            module_path = os.path.join(path, name)
            if not os.path.isfile(module_path):
                continue
            elif not module_path.endswith(('.py', '.pyx')):
                continue
            elif name.startswith(('.', '_')):
                continue
            name = name.rpartition('.py')[0]
            module_path = '.'.join((
                package_path,
                name,
                ))
            module = __import__(module_path, fromlist=['*'])
            if name in dir(module):
                namespace[name] = getattr(module, name)
        if 'system' in namespace:
            del(namespace['system'])
        if ImportManager.__name__ in namespace:
            del(namespace[ImportManager.__name__])

    @staticmethod
    def import_public_names_from_path_into_namespace(
        path,
        namespace,
        delete_system=True,
        ignored_names=None,
        ):
        """
        Inspects the top level of ``path``;
        does not inspect lower levels of path.

        Finds .py modules in path;
        imports public functions from .py modules into namespace;
        imports eponymous datum from .py modules into namespace.

        Find packages in path;
        imports package names into namespace;
        does not import package content into namespace.
        """
        if isinstance(namespace, types.ModuleType):
            namespace = namespace.__dict__
        package_path = ImportManager._split_package_path(path)
        for element in sorted(os.listdir(path)):
            if ignored_names and element in ignored_names:
                continue
            if os.path.isfile(os.path.join(path, element)):
                if element.startswith('_'):
                    continue
                if not element.endswith(('.py', '.pyx')):
                    continue
                # import functions inside module
                name = os.path.splitext(element)[0]
                submodule_name = os.path.join(package_path, name)
                functions = ImportManager._get_public_function_names_in_module(
                    submodule_name)
                for function in functions:
                    # handle public function decorated with @require
                    if function.__name__ == 'wrapper':
                        name = function.func_closure[1].cell_contents.__name__
                    else:
                        name = function.__name__
                    namespace[name] = function
                # import eponymous datum inside module
                submodule_name = submodule_name.replace(os.sep, '.')
                submodule = __import__(submodule_name, fromlist=['*'])
                submodule_dictionary = vars(submodule)
                if name in submodule_dictionary:
                    value = submodule_dictionary[name]
                    namespace[name] = value
            elif os.path.isdir(os.path.join(path, element)):
                if element not in ('.svn', '.git', 'test', '__pycache__'):
                    initializer_file_path = os.path.join(
                        path,
                        element,
                        '__init__.py',
                        )
                    if os.path.exists(initializer_file_path):
                        submod = '.'.join((package_path, element))
                        namespace[element] = __import__(submod, fromlist=['*'])
            else:
                message = 'neither a directory or file: {!r}'
                message = message.format(element)
                raise ImportError(message)
        ImportManager._import_contents_of_public_packages_in_path_into_namespace(
            path, namespace)
        if delete_system:
            if 'system' in namespace:
                del(namespace['system'])
        if ImportManager.__name__ in namespace:
            del(namespace[ImportManager.__name__])

    @staticmethod
    def import_structured_package(
        path,
        namespace,
        delete_system=True,
        ignored_names=None,
        ):
        """
        Imports public names from ``path`` into ``namespace``.

        This is the custom function that all Abjad packages use to import
        public classes and functions on startup.

        The function will work for any package laid out like Abjad packages.
        """
        ImportManager.import_public_names_from_path_into_namespace(
            path,
            namespace,
            delete_system=delete_system,
            ignored_names=ignored_names,
            )
        if delete_system:
            if 'system' in namespace:
                del(namespace['system'])
        if ImportManager.__name__ in namespace:
            del(namespace[ImportManager.__name__])
